 ; Ŀ
 ;   Cabx - attach extended data to an entity.                             
 ;   Copyright 2002, 2011, 2012, 2018 by Rocket Software Ltd.              
 ;                                                                         
 ;   Contains:                                                             
 ;   Xcab - Tag an entity, add it to the database.  The main function.     
 ;   Xdash - Tag a block, split the database string at dashes.             
 ;   Xxdash - Tag lots of blocks, split the database string at dashes.     
 ;   Upcx - update tagged textlike entities from the database file.        
 ;   Idf  - identify entity/database mismatches but don't update.          
 ;   Idt  - mark all entities which should be tagged but aren't - their    
 ;          strings are in the database but they don't have xdata.         
 ;   Idx  - identify all tagged entities.                                  
 ;   Idux - identify all untagged entities.                                
 ;   Igx  - find all blocks with xdata, mark similar ones without.         
 ;   Indx - change an index tag in an entity.                              
 ;   Showx - read the xdata value from an entity.                            
 ;   Sux  - suck xdata and a value from another entity.                    
 ;   Upx  - change the string in an entity and change the string value     
 ;          associated with its index tag in the database.                 
 ;   Wrx  - Overwrite values in the database with those in the drawing.    
 ;   Xxall - write selected entities to a database file.                   
 ; 

 ; Ŀ
 ;   How this thing works, and some terminology.                           
 ;   (Made-up terms aren't a law of nature, feel free to use your own.)    
 ;   Cabx adds extended entity data (xdata) to text and attributes.        
 ;   This xdata contains an Index Tag which is saved in a Database File    
 ;   on the same line as the string value which the text or attribute      
 ;   should have.                                                          
 ;   The string to be put into the text or attribute (which you have to    
 ;   input) is checked against the database file.  If it's there then      
 ;   the matching index tag is added to the entity xdata, if not then an   
 ;   unused index string is read from the Index Source File, added to      
 ;   the entity as xdata, and added to the database file on a line with    
 ;   the string value.                                                     
 ;   When Upcx is run it finds the text and attributes with xdata and      
 ;   checks the index strings against the database file.  If the string    
 ;   value associated with that index tag in the database file is          
 ;   different from the one in the text or attribute then the entity is    
 ;   updated.                                                              
 ;   Thus you can change a value throughout an entire package by           
 ;   editing the database file and running Upcx either from a batch        
 ;   or as each drawing is opened.                                         
 ; 

 ; Ŀ
 ;   File and Entity Subroutines:                                          
 ; Ĵ
 ;   A22oc    Assoc by position in the sublist.                           
 ;   Addcb    Add a new line to a dbase file.                             
 ;   Admult   Add one of more new lines to a dbase file.                  
 ;   Adxd     Add or replace extended entity data.                        
 ;   Alphox   Find the sublist with the alph. first leading string.       
 ;   Binx     Split a string at dashes, install into attributes.          
 ;   Dbrep    Replace the value assoc. with an index tag in a database.   
 ;   Getals   Get all strings from text and blocks.                       
 ;   Getbac   Find the line in a file after the one matching a string.    
 ;   Getcab   Read the database file into a list of lists.                
 ;   Getxd    Get the extended data for an app from an entity.            
 ;   Horizf   Put a list of lists in alphabetical order by first element. 
 ;   Lizx     Make xdata into a list.                                     
 ;   Prelp    Divide a text string into a list of substrings              
 ;   Pumul    Attach xdata index tags to multiple entities.               
 ;   Spit     Find the insertion point of a text entity.                  
 ;   Splat    Divide a text string into a list of substrings.             
 ;   Upex     Update the extended data for a single entity.               
 ;   Valp     Put text into an entity.                                    
 ;   Xdash    Tag a block, split the database string at dashes.           
 ; 

 ; Ŀ
 ;   Graphic Subroutines:                                                  
 ; Ĵ
 ;   Ccstar   Draw an individual grstar (centred).                        
 ;   CI       Grdraw circle maker.                                        
 ;   Cstar    Draw an individual grstar (centred).                        
 ;   Cvo      Mark a point passed as the only argument.                   
 ;   Ici      Grdraw circle maker.                                        
 ;   Mark     Mark a point.                                               
 ;   Sta      Draw an individual grstar                                   
 ;   Olaf     Make a ring of stars.                                       
 ; 

 ; Ŀ
 ;   Subroutines.                                                          
 ; 

 ; Ŀ
 ;   A22oc - assoc by position (originally second position.)               
 ;   Arguments: Str, the string to match.                                  
 ;              Malist, the master list.                                   
 ;              Pos, the zero based position to search in.                 
 ;   Returns a (possibly nil) list of matching sublists.                   
 ; 
 (DEFUN A22OC (str malist pos / num frond sub)
  (setq num 0)
  (while (and malist (setq sub (nth num malist))) ; don't crash on null malist
         (if (= (nth pos sub) str)
             (setq frond (cons sub frond)))
         (setq num (1+ num)))
 frond)
 ; Ŀ
 ;   A22oc end.                                                            
 ; 

 ; Ŀ
 ;   Addcb - add a new line to a dbase file.                               
 ;   Arguments: Idx, the new index string.                                 
 ;              Tagg, the new tagg string.                                 
 ;              Fnam, the file name.                                       
 ;   Calls nothing, Returns nothing.                                       
 ; 
 (DEFUN ADDCB (idx tagg fnam / fn)
 ; Ŀ
 ;   Open the data file and make the configuration list.                   
 ; 
  (if (setq fn (open fnam "a"))
      (progn
           (while (< (strlen idx) 8)
                  (setq idx (strcat idx " ")))
           (write-line (strcat idx " |  " tagg) fn)
           (close fn)))
 (princ))
 ; Ŀ
 ;   Addcb end.                                                            
 ; 

 ; Ŀ
 ;   Admult - add one or more new lines to a dbase file.                   
 ;   Arguments: Liss, a list: ((new_index_string  new_tagg_string) ...)    
 ;              Fnam, the file name.                                       
 ;   Calls nothing, Returns nothing.                                       
 ; 
 (DEFUN ADMULT (liss fnam / fn num sub idx tagg)
  (if (and liss
           (setq fn (open fnam "a")))
      (progn
           (setq num 0)
           (while (setq sub (nth num liss))
                  (setq num (1+ num))
                  (setq idx (car sub))
                  (setq tagg (cadr sub))
                  (while (< (strlen idx) 8)
                         (setq idx (strcat idx " ")))
                  (write-line (strcat idx " |  " tagg) fn))
           (close fn)))
 (princ))
 ; Ŀ
 ;   Admult end.                                                           
 ; 

 ; Ŀ
 ;   Subroutine Adxd - add or replace extended entity data.                
 ;   Arguments: Enam, the entity ename.                                    
 ;              Dalis, the data string and bracket list.                   
 ;              Apnam, the program name.                                   
 ;   Returns T if everything worked, nil otherwise.                        
 ; 
 (DEFUN ADXD (enam dalist apnam / bb exdat exx sub ndata entt modok enam parok)
 ; Ŀ
 ;   Make sure that AutoCAD knows which program is talking to it.          
 ; 
  (if (not (tblsearch "appid" apnam))
      (regapp apnam))
 ; Ŀ
 ;   See if the entity has any xdata belonging to this program.            
 ; 
  (setq bb (entget enam (list apnam)))
  (setq exdat (cdadr (setq exx (assoc -3 bb))))
 ; Ŀ
 ;   Make the new xdata list, 1 sublist for each string in the list Data.  
 ; 
  (while (setq sub (car dalist))
         (setq dalist (cdr dalist))
         (if (member sub '("{" "}"))
             (setq ndata (append ndata (list (cons 1002 sub))))
             (setq ndata (append ndata (list (cons 1000 sub))))))
  (setq ndata (list -3 (append (list apnam) ndata)))
 ; Ŀ
 ;   If there was existing xdata then install the new list in its place,   
 ;   otherwise add the new list to the entity data as xdata.               
 ; 
  (if exdat
      (setq entt (subst ndata exx bb))
      (setq entt (append (entget enam) (list ndata))))
 ; Ŀ
 ;   Update the entity, return T if it worked, otherwise nil.              
 ; 
  (setq modok (entmod entt))
 ; Ŀ
 ;   If the entity was an attribute then get the parent entity name        
 ;   and attach a marker to it so that an ssget can find it.               
 ;   Recurse, but just once.                                               
 ; 
  (if (= (cdr (assoc 0 entt)) "ATTRIB")
      (progn
           (setq enam (cdr (assoc 330 entt)))
           (setq ndata (list "null"))
           (setq parok (adxd enam ndata apnam))
           (setq modok (and modok parok))))
 ; Ŀ
 ;   Return t or nil - it worked or didn't.                                
 ; 
  (if modok T ()))
 ; Ŀ
 ;   Adxd end.                                                             
 ; 

 ; Ŀ
 ;   Subroutine Alpho - find the sublist in which the leading string is    
 ;   first in alphabetical order.  Numbers are converted to strings so     
 ;   that the > function doesn't crash.                                    
 ;   Argument: Nexlst, a list.                                             
 ;   Typically called by Horize.                                           
 ;   Calls nothing, returns a string.                                      
 ;   This is a version of Alpho which ditches leading ; substrings from    
 ;   sublists which are lists so that commented sublists can be handled.   
 ; 
 (DEFUN ALPHOX (nexlst / num neth first)
  (setq num 0)
  (while (setq neth (nth num nexlst))
         (cond ((and (= (type neth) 'LIST)
                     (= substr 1 1 (car neth) ";"))
                (setq neth (cadr neth)))
               ((= (type neth) 'LIST)
                (setq neth (car neth))))
         (cond ((= (type neth) 'INT)
                (setq neth (itoa neth)))
               ((= (type neth) 'REAL)
                (setq neth (rtos neth 2 6))))
         (if (or (null first) (< neth first))
             (setq first neth))
         (setq num (1+ num)))
 first)
 ; Ŀ
 ;   Alphox end.                                                           
 ; 

 ; Ŀ
 ;   Atlist - suck attribute values from a block into a list.              
 ; 
 (DEFUN ATLIST (enam / entt tagg taglst)
  (while (and (setq entt (entget (setq enam (entnext enam))))
              (/= (cdr (assoc 0 entt)) "SEQEND"))
         (setq tagg (cdr (assoc 1 entt)))
         (setq taglst (append taglst (list tagg))))
 taglst)
 ; Ŀ
 ;   Atlist end.                                                           
 ; 

 ; Ŀ
 ;   Blix - split a string at dashes, put into a block.                    
 ;   Arguments: Enam, the block insertion ename.                           
 ;              Vlastr, the string.                                        
 ;   Stops when it runs put of attributes or substrings.                   
 ;   Assumes the block has attributes.                                     
 ;   Calls Splat.                                                          
 ;   Returns a black velvet painting you'd happily forgotten about.        
 ; 
 (DEFUN BLIX (enam vlastr / vlasta gnuval entt)
  (setq vlasta (splat "-" vlastr))
  (while (and (setq gnuval (car vlasta))
              (/= "SEQEND" (cdr (assoc 0 (setq entt (entget
                                               (setq enam (entnext enam))))))))
         (setq vlasta (cdr vlasta))
         (entmod (subst (cons 1 gnuval) (assoc 1 entt) entt)))
 (princ))
 ; Ŀ
 ;   Blix end.                                                             
 ; 

 ; Ŀ
 ;   Subroutine Ccstar - draw an individual grstar (centred).              
 ;   Takes four arguments: centre point, side length, rotation (radians),  
 ;   and colour.  Returns nothing, but draws a star.                       
 ; 
 (DEFUN CCSTAR (pa sidlen rota colo / anginc angg hafang pb)
  (setq px pa)
  (setq pa (polar pa (+ rota (/ pi 2)) (* sidlen 1.37638192)))
  (grdraw pa px 5)
  (setq anginc (* 1.6 pi))
  (setq angg (+ rota (* 1.6 pi)))
  (setq hafang (* 0.8 pi))
  (repeat 5
         (setq pb (polar pa angg sidlen))
         (grdraw pb px 2)
         (grdraw pa pb colo)
         (setq angg (- angg anginc))
         (setq pa pb)
         (setq pb (polar pa angg sidlen))
         (grdraw pb px 2)
         (grdraw pa pb colo)
         (setq angg (- angg hafang))
         (setq pa pb))
 (princ))
 ; Ŀ
 ;   Subroutine Ccstar end.                                                
 ; 

 ; Ŀ
 ;   Subroutine CI - grdraw circle maker.                                  
 ; 
 (DEFUN CI (pa radd colo / reps pa pa1 pa2 angg)
  (setq reps 32)
  (setq angg 0)
  (setq incr (/ pi (/ reps 2)))
  (setq pa1 (polar pa angg radd))
  (repeat reps
          (setq angg (+ angg incr))
          (setq pa2 (polar pa angg radd))
          (grdraw pa1 pa2 colo)
          (setq pa1 pa2))
 (princ))
 ; Ŀ
 ;   Ci end.                                                               
 ; 

 ; Ŀ
 ;   Subroutine Cstar - draw an individual grstar (centred).               
 ;   Takes four arguments: centre point, side length, rotation (radians),  
 ;   and colour.  Returns nothing, but draws a star.                       
 ; 
 (DEFUN CSTAR (pa sidlen rota colo / anginc angg hafang pb)
  (setq px pa)
  (setq pa (polar pa (+ rota (/ pi 2)) (* sidlen 1.37638192)))
  (setq anginc (* 1.6 pi))
  (setq angg (+ rota (* 1.6 pi)))
  (setq hafang (* 0.8 pi))
  (repeat 5
         (setq pb (polar pa angg sidlen))
         (grdraw pa pb colo)
         (setq angg (- angg anginc))
         (setq pa pb)
         (setq pb (polar pa angg sidlen))
         (grdraw pa pb colo)
         (setq angg (- angg hafang))
         (setq pa pb))
 (princ))
 ; Ŀ
 ;   Subroutine Cstar end.                                                 
 ; 
         
 ; Ŀ
 ;   Cvo - mark a point passed as the only argument.                       
 ; 
 (DEFUN CVO (pa rad colo / reps pa angg incr)
  (setq reps 32)
  (setq angg (/ pi 4))
  (setq incr (* 2.5 (/ pi reps)))
  (setq crad (/ rad 2))
 ; Ŀ
 ;   The main loop.                                                        
 ; 
  (repeat reps
          (setq rad (* rad 1.1))
          (setq pb (polar pa angg rad))
          (ici pb crad 16 colo)
          (setq pb (polar pa angg (- rad)))
          (ici pb crad 16 colo)
          (setq crad (* crad 0.95))
          (setq angg (+ angg incr)))
 (princ))
 ; Ŀ
 ;   Cvo end.                                                              
 ; 

 ; Ŀ
 ;   Dbrep - replace the value associated with an index tag in a dbase     
 ;   file.                                                                 
 ;   Arguments: Idx, the index tag string.                                 
 ;              Tagg, the new value tag string.                            
 ;              Fnam, the file name.                                       
 ;   Calls Splat, Returns nothing.                                         
 ; 
 (DEFUN DBREP (idx tagg fnam / fn linn0 linn comm gnustr sub malist num)
 ; Ŀ
 ;   Open the data file and make the configuration list.                   
 ; 
  (if (setq fn (open fnam "r"))
      (progn
           (while (setq linn0 (read-line fn))
 ; Ŀ
 ;   If the first substring is the index tag, remake the string.           
 ; 
                  (if (and (/= linn0 "")
                           (= (car (splat "|" linn0)) idx))
                      (progn
 ; Ŀ
 ;   Save the comment, if there is one.                                    
 ; 
                           (setq linn (splat ";" linn0))
                           (if (> (length linn) 1)
                               (progn
                                    (setq comm (last linn))
                                    (setq linn0 (car linn))))
 ; Ŀ
 ;   Split the remainder of the string at |.                               
 ; 
                           (setq linn (splat "|" linn0))
 ; Ŀ
 ;   Make a new list - swap out the tag.                                   
 ; 
                           (setq linn (append (list (car linn) tagg)
                                              (cddr linn)))
 ; Ŀ
 ;   Make the list back into a string.                                     
 ; 
                           (setq gnustr "")
                           (while (setq sub (car linn))
                                  (setq linn (cdr linn))
                                  (while (< (strlen sub) 8)
                                         (setq sub (strcat sub " ")))
                                  (if (/= gnustr "")
                                      (setq gnustr (strcat gnustr " |  " sub))
                                      (setq gnustr sub)))
 ; Ŀ
 ;   Add the comment, if there was one.                                    
 ; 
                           (if comm
                               (setq gnustr (strcat gnustr "   ; " comm)))
 ; Ŀ
 ;   Add the new string to the string list.                                
 ; 
                           (setq malist (cons gnustr malist)))
 ; Ŀ
 ;   Else add the existing string to the string list.                      
 ; 
                      (setq malist (cons linn0 malist))))
 ; Ŀ
 ;   Write the list back to the file.                                      
 ; 
           (close fn)
           (setq malist (reverse malist))
           (setq num 0)
           (if (setq fn (open fnam "w"))
               (progn
                    (while (setq sub (nth num malist))
                           (setq num (1+ num))
                           (write-line sub fn))
                    (close fn)))))
 (princ))
 ; Ŀ
 ;   Dbrep end.                                                            
 ; 

 ; Ŀ
 ;   Getals - get all strings from text and blocks.                        
 ;   Arguments: Ss, a selection set of text and attributes.                
 ;   Calls nothing.                                                        
 ;   Returns a list in alphabetical order without duplicates.              
 ; 
 (DEFUN GETALS (ss / num enam typp entt stra malist)
  (setq num 0)
  (while (and ss (setq enam (ssname ss num)))
         (setq num (1+ num))
 ; Ŀ
 ;   Get the entity type, only work with text and attributes.              
 ; 
         (setq typp (cdr (assoc 0 (setq entt (entget enam)))))
         (cond ((= typp "TEXT")
                (setq stra (cdr (assoc 1 entt)))
                (if (not (member stra malist))
                    (setq malist (cons stra malist))))
               ((and (= typp "INSERT") (= (cdr (assoc 66 entt)) 1))
                (while (/= "SEQEND" (cdr (assoc 0 (setq entt (entget
                                                (setq enam (entnext enam)))))))
                       (setq stra (cdr (assoc 1 entt)))
                       (if (not (member stra malist))
                           (setq malist (cons stra malist)))))))
 (if malist (tize malist 1 t)))
 ; Ŀ
 ;   Getals end.                                                           
 ; 

 ; Ŀ
 ;   Getbac - find the line in a file after the one matching a string.     
 ;   Arguments: Filnam, the file name.                                     
 ;              Stra, the string.                                          
 ;   Returns the configuration list.                                       
 ; 
 (DEFUN GETBAC (filnam stra / fn str stop gnu)
 ; Ŀ
 ;   Open the data file and read lines until a matching one is found.      
 ; 
  (cond ((null (setq fn (open filnam "r")))
         (prompt "\n*** Index file read error. ***"))
        ((null stra)
         (setq gnu (read-line fn)))
        (t
         (while (and (null stop)
                     (setq str (read-line fn)))
                (if (= str stra)
                    (setq stop t)))
                (setq gnu (read-line fn))))
  (if fn (close fn))
  (if (null gnu) (prompt "\n*** Index string not available. ***"))
 gnu)
 ; Ŀ
 ;   Getbac end.                                                           
 ; 

 ; Ŀ
 ;   Getcab - read the xdata and string file into a list of lists.         
 ;   Removes comments and leading and trailing spaces, splits the          
 ;   remainder at vertical bars: |.                                        
 ;   Arguments: Cfgnam, the configuration file or nil.  If the file        
 ;                      can't be found or the argument is nil then         
 ;                      Gatcab returns nil.                                
 ;   Returns the configuration list or nil.                                
 ; 
 (DEFUN GETCAB (cfgnam / fn str len liss cfglst)
 ; Ŀ
 ;   Open the data file and make the configuration list.                   
 ; 
  (if (and cfgnam
           (setq fn (open cfgnam "r")))
      (progn
           (while (setq str (read-line fn))
                  (while (= (substr str 1 1) " ")
                         (setq str (substr str 2)))
                  (if (not (member (substr str 1 1) '(";" "")))
                      (progn
                           (setq str (car (splat ";" str)))
                           (while (= (substr str (setq len (strlen str))) " ")
                                  (setq str (substr str 1 (1- len))))
                           (setq liss (splat "|" str))
                           (setq cfglst (cons liss cfglst)))))
           (close fn)))
 cfglst)
 ; Ŀ
 ;   Getcab end.                                                           
 ; 

 ; Ŀ
 ;   Subroutine Getxd - get the extended data for an app from an entity.   
 ;   Arguments: Enam, the entity ename.                                    
 ;              Apnam, the program name.                                   
 ;   Returns a list of values, with { and } removed.                       
 ; 
 (DEFUN GETXD (enam apnam / enttx ndata)
 ; Ŀ
 ;   See if the entity has any xdata belonging to this program.            
 ; 
  (setq enttx (entget enam (list apnam)))
  (if (assoc -3 enttx)
      (setq ndata (lizx enttx)))
 ndata)
 ; Ŀ
 ;   Getxd end.                                                            
 ; 

 ; Ŀ
 ;   Subroutine Horizf - put a list in alphabetical order.  If an element  
 ;   is a list it is alphabetized by first element, numbers are converted  
 ;   to strings.                                                           
 ;   Arguments: Nexlst, a list.                                            
 ;   Calls Alphox, returns the list, reordered.                            
 ;   This is a variant of Horize.  This version uses the second element    
 ;   as the index string if the first begins with ; so that commented      
 ;   and flagged lines can still be indexed.                               
 ; 
 (DEFUN HORIZF (nexlst / low sub neth hrzlst newlst orderd)
  (while nexlst
        (setq low (alphox nexlst))                   ; lowest leading number
        (while (and nexlst (setq sub (nth 0 nexlst)))
               (cond ((and (= (type sub) 'LIST)
                           (= substr 1 1 (car sub) ";"))
                      (setq neth (cadr sub)))
                     ((= (type sub) 'LIST)
                      (setq neth (car sub))))
               (cond ((= (type neth) 'INT)
                      (setq neth (itoa neth)))
                     ((= (type neth) 'REAL)
                      (setq neth (rtos neth 2 6))))
               (if (equal low neth)
                   (setq hrzlst (append hrzlst (list sub)))
                   (setq newlst (append newlst (list sub))))
               (setq nexlst (cdr nexlst)))          ; remove 1st ent from list
        (setq orderd (append orderd hrzlst))        ; add lev sublst to levels
        (setq hrzlst ())                            ; set to () for next loop
        (setq nexlst newlst)                        ; nexlst reconstituted
        (setq newlst ()))                           ; empty new list & reuse
  orderd)
 ; Ŀ
 ;   Horizf end.                                                           
 ; 

 ; Ŀ
 ;   Subroutine Ici - grdraw circle maker.                                 
 ; 
 (DEFUN ICI (pa radd reps colo / angg incr pa1 pa2)
  (setq angg 0)
  (setq incr (/ pi (/ reps 2)))
  (setq pa1 (polar pa angg radd))
  (repeat reps
          (setq angg (+ angg incr))
          (setq pa2 (polar pa angg radd))
          (grdraw pa1 pa2 colo)
          (setq pa1 pa2))
 (princ))
 ; Ŀ
 ;   Ici end.                                                              
 ; 

 ; Ŀ
 ;   Listi - make a list of strings into one string.                       
 ;   Arguments: Alist, a list of strings.                                  
 ;              Sepstr, the separator string.                              
 ;   Returns a string.                                                     
 ; 
 (DEFUN LISTI (alist sepstr / thestr len)
  (setq thestr "")
 ; Ŀ
 ;   You don't ever really have to have mapcar, but it is nice...          
 ; 
  (mapcar '(lambda (astr)
                   (if (= (type astr) 'INT) (setq astr (itoa astr)))
                   (setq thestr (strcat thestr sepstr astr)))
                    alist)
 ; Ŀ
 ;   Remove the extraneous copy of sepstr from the string end.             
 ; 
  (if (> (strlen thestr) (setq len (strlen sepstr)))
      (setq thestr (substr thestr (1+ len))))
 thestr)
 ; Ŀ
 ;   Listi end.                                                            
 ; 

 ; Ŀ
 ;   Subroutine Lizx - make xdata into a list.                             
 ;   Arguments: Ett, the entity data list with xdata.                      
 ;   Calls nothing, Returns a list of strings.                             
 ;   Isn't all that elegant.                                               
 ; 
 (DEFUN LIZX (entt / cralp num subx xdal)
  (setq cralp (cdadr (assoc -3 entt)))
  (setq num 0)
  (while (setq subx (nth num cralp))
         (setq num (1+ num))
         (if (= (car subx) 1000)
             (setq xdal (cons (cdr subx) xdal))))
 (reverse xdal))
 ; Ŀ
 ;   Subroutine Lizx end                                                   
 ; 

 ; Ŀ
 ;   Mark - mark a point.                                                  
 ;   Arguments: Pa - the point to mark.                                    
 ;              Rad - the marker segment length.                           
 ;              Colo - the marker grdraw line colour.                      
 ; 
 (DEFUN MARK (pa rad colo /)
  (grdraw (polar pa (/ pi 4) rad) (polar pa (* 1.25 pi) rad) colo)
  (grdraw (polar pa (* pi 0.75) rad) (polar pa (* pi 1.75) rad) colo)
 (princ))
 ; Ŀ
 ;   Mark end.                                                             
 ; 

 ; Ŀ
 ;   Olaf - make a ring of stars.                                          
 ;   Arguments: Pa, the circle centre.                                     
 ;              Rad, the circle radius.                                    
 ;              Stsz, star size as a fraction of circle radius.            
 ;              Angg, the starting circle rotation.                        
 ;              Colo, the starting star colour.                            
 ;              Numa, the number of stars.                                 
 ;   Calls Sta.  Returns nothing.                                          
 ; 
 (DEFUN OLAF (pa rad stsz angg colo numa / colo1 vsize angg rad pa2)
  (setq colo1 colo)
  (repeat numa
         (setq pa2 (polar pa angg rad))
         (sta pa2 (* rad stsz) (- angg (/ pi 2)) colo)
         (if (> colo (+ colo1 8))
             (setq colo colo1)
             (setq colo (1+ colo)))
         (setq angg (+ angg (/ pi numa 0.5))))
 (princ))
 ; Ŀ
 ;   Olaf end.                                                             
 ; 

 ; Ŀ
 ;   Subroutine Prelp - divide a text string into a list of substrings     
 ;   at any character in a list.                                           
 ;   Arguments: Seplst, the list of field separator characters.            
 ;              Linn, the text string.                                     
 ;   Returns a list of field values, removes leading and trailing spaces.  
 ; 
 (DEFUN PRELP (seplst linn / pos len name1 strlst)
  (while (/= (strlen linn) 0)
 ; Ŀ
 ;   Find the first separator character, save everything before it into    
 ;   the Name1 variable, remove it from the start of the penis Linn.       
 ; 
         (setq pos 1)
         (setq len (strlen linn))
         (while (and (not (member (substr linn pos 1) seplst))
                     (>= len pos))
                (setq pos (1+ pos)))
         (setq name1 (substr linn 1 (1- pos)))
         (setq linn (substr linn (1+ pos)))
 ; Ŀ
 ;   Remove spaces from the front and back of Name1.                       
 ; 
         (while (and (> (strlen name1) 0)
                     (= (substr name1 (setq len (strlen name1))) " "))
                (setq name1 (substr name1 1 (1- len))))
         (while (and (> (strlen name1) 0)
                     (= (substr name1 1 1) " "))
                (setq name1 (substr name1 2)))
 ; Ŀ
 ;   Add Name1 to the substring list Strlst.                               
 ; 
         (setq strlst (append strlst (list name1))))
 ; Ŀ
 ;   If the string contained no separator characters then Strlst will be   
 ;   nil, so return a list containing the original string.                 
 ; 
  (if (null strlst) (setq strlst (list linn)))
 strlst)
 ; Ŀ
 ;   Prelp end.                                                            
 ; 

 ; Ŀ
 ;   Pumul - attach xdata index tags to multiple entities.                 
 ;   Arguments: Ss, a selection set of text and attributes.                
 ;              Taglis, a list: ((index string) ...)                       
 ;              Apnam, the application name for the xdata.                 
 ;   Calls nothing.                                                        
 ;   Returns nothing.                                                      
 ; 
 (DEFUN PUMUL (ss taglis apnam / num enam typp entt stra ndxi)
  (setq num 0)
  (while (and ss (setq enam (ssname ss num)))
         (setq num (1+ num))
 ; Ŀ
 ;   Get the entity type, only work with text and attributes.              
 ; 
         (setq typp (cdr (assoc 0 (setq entt (entget enam)))))
         (cond ((= typp "TEXT")
                (setq stra (cdr (assoc 1 entt)))
                (setq ndxi (caar (a22oc stra taglis 1)))
                (adxd enam (list "{" ndxi "1" "}") apnam))
               ((and (= typp "INSERT") (= (cdr (assoc 66 entt)) 1))
                (while (/= "SEQEND" (cdr (assoc 0 (setq entt (entget
                                                (setq enam (entnext enam)))))))
                       (setq stra (cdr (assoc 1 entt)))
                       (setq ndxi (caar (a22oc stra taglis 1)))
                       (adxd enam (list "{" ndxi "1" "}") apnam)))))
 (princ))
 ; Ŀ
 ;   Pumul end.                                                            
 ; 

 ; Ŀ
 ;   Spit - returns the insertion point of the text entity whose data was  
 ;   passed as its sole argument.  Note that this is not necessarily the   
 ;   same as the 10 association code.                                      
 ;   Latest version: February 20th 2011.                                   
 ; 
 (DEFUN SPIT (entt / xjust yjust)
  (setq xjust (cdr (assoc 72 entt)))
  (if (member (cdr (assoc 0 entt)) '("ATTDEF" "ATTRIB"))
      (setq yjust (cdr (assoc 74 entt)))
      (setq yjust (cdr (assoc 73 entt))))
  (if (or (/= xjust 0) (/= yjust 0))
      (cdr (assoc 11 entt))
      (cdr (assoc 10 entt))))
 ; Ŀ
 ;   Spit end.                                                             
 ; 

 ; Ŀ
 ;   Subroutine Splat - divide a text string into a list of substrings.    
 ;   Arguments: Sepchr, the field separator character.                     
 ;              Linn, the text string.                                     
 ;   Returns a list of field values, removes leading and trailing spaces.  
 ;                                                                         
 ;   Completely rewritten 11.19.2000.                                      
 ; 
 (DEFUN SPLAT (sepchr linn / pos len name1 strlst)
  (while (/= (strlen linn) 0)
 ; Ŀ
 ;   Find the first separator character, save everything before it into    
 ;   the Name1 variable, remove it from the start of the penis Linn.       
 ; 
         (setq pos 1)
         (setq len (strlen linn))
         (while (and (/= (substr linn pos 1) sepchr)
                     (>= len pos))
                (setq pos (1+ pos)))
         (setq name1 (substr linn 1 (1- pos)))
         (setq linn (substr linn (1+ pos)))
 ; Ŀ
 ;   Remove spaces from the front and back of Name1.                       
 ; 
         (while (and (> (strlen name1) 0)
                     (= (substr name1 (setq len (strlen name1))) " "))
                (setq name1 (substr name1 1 (1- len))))
         (while (and (> (strlen name1) 0)
                     (= (substr name1 1 1) " "))
                (setq name1 (substr name1 2)))
 ; Ŀ
 ;   Add Name1 to the substring list Strlst.                               
 ; 
         (setq strlst (append strlst (list name1))))
 ; Ŀ
 ;   If the string contained no separator characters then Strlst will be   
 ;   nil, so return a list containing the original string.                 
 ; 
  (if (null strlst) (setq strlst (list linn)))
 strlst)
 ; Ŀ
 ;   Splat end.                                                            
 ; 

 ; Ŀ
 ;   Subroutine Sta - draw an individual grstar                            
 ;   Takes four arguments: top, side length, rotation (radians),           
 ;   and colour.  Returns nothing, but draws a star.                       
 ; 
 (DEFUN STA (pa sidlen rota colo / anginc angg hafang pb)
  (setq anginc (* 1.6 pi))
  (setq angg (+ rota (* 1.6 pi)))
  (setq hafang (* 0.8 pi))
  (repeat 5
         (setq pb (polar pa angg sidlen))
         (grdraw pa pb colo)
         (setq angg (- angg anginc))
         (setq pa pb)
         (setq pb (polar pa angg sidlen))
         (grdraw pa pb colo)
         (setq angg (- angg hafang))
         (setq pa pb))
 (princ))
 ; Ŀ
 ;   Subroutine Sta end.                                                   
 ; 
 ; Ŀ
 ;   Locar - find the string with the lowest character at position Pos.    
 ;   Check each string, compare its substr pos character to the previous   
 ;   one. If it's lower, it becomes the new first one.                     
 ;   Optionally Strcase the comparison characters so it's not case         
 ;   sensitive.                                                            
 ;   Arguments: Listr, a list of strings.                                  
 ;              Pos, the compare position.                                 
 ;              Casa, T = case sensitive, nil = not.                       
 ;   Returns a list: (Lowest_string  Rest_of_list)                         
 ;   Note that a list may contain duplicate strings.                       
 ;   Will crash on an empty list.                                          
 ;   ("aaa") returns ("aaa" nil), which is what it should.                 
 ; 
 (defun locar (listr pos casa / num suba chra iowa curchr)
  (setq num 0)
  (while (setq suba (nth num listr))
         (setq num (1+ num))
         (setq chra (substr suba pos 1))
         (if casa (setq chra (strcase chra)))
         (if (or (null curchr)
                 (< chra curchr))
             (progn
                  (setq iowa suba)
                  (setq curchr chra))))
  (setq listr (remove iowa listr))
 (list iowa listr))
 ; Ŀ
 ;   Locar end.                                                            
 ; 

 ; Ŀ
 ;   Remove - remove the first occurrence of an element from a list.       
 ;   Arguments: Remora, the element to remove.                             
 ;              Lista, a list.                                             
 ;   Returns a list, without Remora if it was found.                       
 ; 
 (DEFUN REMOVE (remora lista / suba gnu)
  (while (setq suba (car lista))
         (setq lista (cdr lista))
         (if (equal suba remora)
             (progn
                  (setq gnu (append gnu lista))
                  (setq lista ()))
             (setq gnu (append gnu (list suba)))))
 gnu)
 ; Ŀ
 ;   Remove end.                                                           
 ; 

 ; Ŀ
 ;   Subroutine Tize - alphabetize a list of strings.                      
 ;   Arguments: Strlis, a list of strings.                                 
 ;              Pos, the 1 based substring position, for recursion.        
 ;                   This should be 1.                                     
 ;              Casa, Case sensitive - t or nil.                           
 ;   Calls Locar.                                                          
 ;   Recursive (2x).                                                       
 ;   Returns an ordered list.                                              
 ; 
 (DEFUN TIZE (strlis pos casa / strls0 curr suba chra gnu lass lassa chra0)
  (setq strls0 strlis)
  (while (and strlis (setq curr (locar strlis pos casa)))
         (setq suba (car curr))                 ; 1st string
         (setq strlis (cadr curr))              ; save the rest back to strlis
         (setq chra (substr suba pos 1))        ; current index character
 ; Ŀ
 ;   If gnu is nil, make it from suba.                                     
 ; 
         (cond ((null gnu)
                (setq gnu (list suba)))
 ; Ŀ
 ;   If the index character is the same as the previous one, add suba to   
 ;   the last list in gnu.                                                 
 ; 
               ((equal chra0 chra)
                (setq gnu (reverse gnu))
                (setq lass (car gnu))
                (if (= (type lass) 'list)
                    (setq lass (append lass (list suba)))
                    (setq lass (list lass suba)))
                (setq gnu (reverse (cons lass (cdr gnu)))))
 ; Ŀ
 ;   If we're starting a new sublist gnu but the previous sublist had >1   
 ;   member, increment the index pos, and recurse to reorder it.           
 ; 
               ((and (not (equal chra0 chra))
                     (= (type (setq lassa (last gnu))) 'list))
                (setq lassa (tize lassa (1+ pos) casa))
                (setq gnu (reverse (cdr (reverse gnu))))
                (setq gnu (append gnu lassa))
                (setq gnu (append gnu (list suba))))
 ; Ŀ
 ;   Otherwise add suba to gnu.                                            
 ; 
               (t
                (setq gnu (append gnu (list suba)))))
         (setq chra0 chra))
 ; Ŀ
 ;   There still might be a sublist to append here ... or to recurse into. 
 ;   Can it be both?                                                       
 ; 
  (if (= (type (setq lassa (last gnu))) 'list)
      (progn
           (if (not (equal lassa strls0))
               (setq lassa (tize lassa (1+ pos) casa)))
           (setq gnu (reverse (cdr (reverse gnu))))
           (setq gnu (append gnu lassa))))
 gnu)
 ; Ŀ
 ;   Tize end.                                                             
 ; 

 ; Ŀ
 ;   Subroutine Upex - update the string in a single entity.               
 ;   Arguments: Enam, the entity ename.                                    
 ;              Cfglst, the index key and string value list.               
 ;              Apnam, the application name.                               
 ;              Rad, the marker size.                                      
 ;              Rota, the marker rotation angle.                           
 ;   Calls Spit, Ccstar, and Lizt.                                         
 ;   Returns T if everything worked, nil otherwise.                        
 ; 
 (DEFUN UPEX (enam cfglst apnam rad rota / entt enttx pa indx vala val0 upd
                                                                        asoc1)
  (setq entt (entget enam))
  (setq enttx (entget enam (list apnam)))
 ; Ŀ
 ;   Upcx processes all attributes from marked blocks, but not all         
 ;   of them have to have xdata, so check.                                 
 ; 
  (if (assoc -3 enttx)
      (progn
           (setq pa (spit entt))
           (ccstar pa rad rota 4)
           (ccstar pa (* rad 0.75) rota 1)
 ; Ŀ
 ;   Get the index tag from the exdata.                                    
 ; 
           (setq indx (car (lizx enttx)))
 ; Ŀ
 ;   Get the matching sublist from the cable list.                         
 ; 
           (setq vala (cadr (assoc indx cfglst)))
 ; Ŀ
 ;   If the value in the text doesn't match this, update it.               
 ; 
           (setq val0 (cdr (setq asoc1 (assoc 1 entt))))
           (if (and vala (/= vala val0))
               (progn
                    (setq upd t)
                    (entmod (subst (cons 1 vala) asoc1 entt))))))
 upd)
 ; Ŀ
 ;   Subroutine Upex end.                                                  
 ; 

 ; Ŀ
 ;   Subroutine Updsh - update the atts in a block from an a-b-c string.   
 ;   Arguments: Enam, the entity ename.                                    
 ;              Cfglst, the index key and string value list.               
 ;              Apnam, the application name.                               
 ;              Rad, the marker size.                                      
 ;              Rota, the marker rotation angle.                           
 ;   Calls Ccstar, and Lizt.                                               
 ;   Returns nothing.                                                      
 ; 
 (DEFUN UPDSH (enam cfglst apnam rad rota / enttx pa indx vala)
  (setq enttx (entget enam (list apnam)))
  (setq pa (cdr (assoc 10 enttx)))
  (ccstar pa rad rota 4)
  (ccstar pa (* rad 0.75) rota 1)
 ; Ŀ
 ;   Get the index tag from the exdata.                                    
 ; 
  (setq indx (car (lizx enttx)))
 ; Ŀ
 ;   Get the matching sublist from the cable list.                         
 ; 
  (setq vala (cadr (assoc indx cfglst)))
 ; Ŀ
 ;   Update the attribute values in the block.                             
 ; 
  (blix enam vala)
 (princ))
 ; Ŀ
 ;   Subroutine Updsh end.                                                 
 ; 

 ; Ŀ
 ;   Upcfg - update or add a value to a configuration list.                
 ;   Arguments: Liss, the master list.                                      
 ;              First, the xdata sublist from the entity.                   
 ;              Second, the second element - the entity string value.       
 ;   Returns the modified list.                                             
 ; 
 (DEFUN UPCFG (liss first second / sub gnusub)
 ; Ŀ
 ;   See if there is a matching sublist in liss.                           
 ; 
  (setq sub (assoc (car first) liss))
 ; Ŀ
 ;   If not then make a new sublist and add it to liss.                    
 ; 
  (cond ((null sub)
         (setq sub (list (car first) second))
         (setq sub (append sub (cdr first)))
         (setq liss (cons sub liss)))
 ; Ŀ
 ;   If there is a matching sublist but the second element doesn't match   
 ;   then add a second sublist with a warning note.                        
 ;   Note that if both elements match then liss will already contain       
 ;   a sublist with both elements so nothing needs to be done.             
 ; 
        ((and sub (/= second (cadr sub)))
;(print first)
;(print second)
         (setq gnusub (list "; *** Duplicate" (car first) second))
         (setq gnusub (append gnusub (cdr first)))
         (setq liss (cons gnusub liss))))
 liss)
 ; Ŀ
 ;   Upcfg end.                                                            
 ; 

 ; Ŀ
 ;   Subroutine Valp - put text into an entity.                            
 ;   Arguments: Rasp, a replacement string.                                
 ;              Enam, an entity or subentity name.                         
 ;   Calls nothing, returns nothing.                                       
 ; 
 (DEFUN VALP (rasp enam / entt)
  (setq entt (entget enam))
  (entmod (subst (cons 1 rasp) (assoc 1 entt) entt))
  (if (= (cdr (assoc 0 entt)) "ATTRIB")
      (entupd (cdr (assoc 330 entt))))
 (princ))
 ; Ŀ
 ;   Subroutine Valp end.                                                  
 ; 

 ; Ŀ
 ;   Subentity Xdash - apply an xdata tag to a block.  This will tell      
 ;   Upcx to split the string from the dbase file at dashes and apply it   
 ;   to attributes in sequence.                                            
 ;   Arguments: Enam, the block entity name.                               
 ;              Multi, if T then this is a multiple selection and the      
 ;                     current values are to be used, otherwise prompt     
 ;                     for a new value.                                    
 ;              Apnam, the application name.                               
 ;   Calls Getcab, Listi, Atlist, Getxd, Sk, Adxd, Blix, and Getbac.       
 ;   Returns nothing.                                                      
 ; 
 (DEFUN XDASH (enam multi apnam / pref fnam cfglst entt stra xdat tagp skulok
                                                     stop pa tagg ndxi gnundx)
 ; Ŀ
 ;   Get the current drawing path.                                         
 ; 
  (setq pref (getvar "dwgprefix"))
 ; Ŀ
 ;   Read the data file into a list.                                       
 ; 
  (setq fnam (strcat pref "cables.txt"))
  (setq cfglst (getcab fnam))
 ; Ŀ
 ;   Get the block data list.                                              
 ; 
  (setq entt (entget enam))
 ; Ŀ
 ;   Make sure it's a block and has attributes.                            
 ; 
  (if (and (= (cdr (assoc 0 entt)) "INSERT")
           (= (cdr (assoc 66 entt)) 1))
 ; Ŀ
 ;   If so then extract all the strings and add dashes between them.       
 ; 
      (setq stra (listi (atlist enam) "-"))
 ; Ŀ
 ;   Else print a warning and quit.                                        
 ; 
      (progn
           (prompt "** That Is Not A Block With Attributes. **")
           (exit)))
 ; Ŀ
 ;   Warn if it already has a tag in xdata.  The user may be               
 ;   trying to change the value instead of the index tag.                  
 ; 
  (setq xdat (getxd enam apnam))
  (cond ((and xdat (equal xdat '("null")))
         (prompt "\n** That Block Has Subentity Tags - Modify with Xcab. **")
         (setq stop t)
         (if (null multi)
             (exit)))
        (xdat
         (setq tagp (cadr (assoc (car xdat) cfglst)))
         (prompt (strcat "\n*** Caution: That entity already has a tag: "
                         (car xdat) "/" tagp ". ***"))
 ; Ŀ
 ;   If the grskull maker can be loaded add a grdraw warning here.         
 ; 
         (if (setq skulok (load "sk" ()))
             (progn
                  (setq pa (cdr (assoc 10 (entget enam))))
                  (sk pa 0.1)))))
 ; Ŀ
 ;   Get the new tag, offer the current combined string as the default.    
 ; 
  (if (null multi)
      (progn
           (setq tagg (getstring (strcat "\nTag <" stra ">: ")))
           (if (= tagg "") (setq tagg stra)))
      (setq tagg stra))
 ; Ŀ
 ;   If the string is in the database then add the correct index tag       
 ;   to the entity as xdata and call valpx to insert the string, because   
 ;   the user may have input a different one from that in the entity.      
 ;   Note that we are doing a search by second element in the assoc list.  
 ; 
  (cond ((and (null stop) cfglst (setq ndxi (caar (a22oc tagg cfglst 1))))
         (adxd enam (list "{" ndxi "1" "}") apnam)
         (if (null multi) (blix enam tagg)))
 ; Ŀ
 ;   If the string is not in the database then call Getbac to get a new    
 ;   index tag, Addcb to add it and the string to the database file, Axdx  
 ;   to add the index tag to the entity as xdata, and Valpx to split the   
 ;   string and put it into the entity, since it may be a new string.      
 ; 
        ((null stop)
         (setq gnundx (getbac (findfile "index.idx") (caar cfglst)))
         (addcb gnundx tagg fnam)
         (adxd enam (list "{" gnundx "1" "}") apnam)
         (if (null multi) (blix enam tagg))))
 (princ))

 ; Ŀ
 ;   Subroutine Xtype - see how attributes are to be applied to a block.   
 ;   Arguments: Enam, the entity ename.                                    
 ;              Apnam, the program name.                                   
 ;   Calls nothing.                                                        
 ;   Returns a string: "none" = no xdata was found.                        
 ;                     "atts" = each attribute will be tagged.             
 ;                     "dash" = split the string from the database at      
 ;                              dashes and apply to attributes in order.   
 ; 
 (DEFUN XTYPE (enam apnam / xdat typx)
  (setq entt (entget enam (list apnam)))
  (setq xdat (cdr (cadadr (assoc -3 entt))))
  (cond ((null xdat)
         (setq typx "none"))
        ((and xdat (equal xdat "null"))
         (setq typx "atts"))
        (xdat
         (setq typx "dash")))
 typx)
 ; Ŀ
 ;   Subroutine Xtype end.                                                 
 ; 

 ; Ŀ
 ;   Xwrite - write a list of strings out to a database file.              
 ;   Arguments: Lista, the list of substring lists.                        
 ;              Fnam, the file name with path.                             
 ;   Calls Nothing, Returns nothing.                                       
 ;   This is designed to be called by Xcab and thus doesn't handle         
 ;   comments - it is a one-way dump of drawing data to a database file    
 ;   for last-ditch situations where the drawings are available but the    
 ;   database file is not.                                                 
 ; 
 (DEFUN XWRITE (lista fnam / fn num linn sub gnustr malist num fnam)
 ; Ŀ
 ;   Make the new database list.                                           
 ; 
  (setq num 0)
  (while (setq linn (nth num lista))
         (setq num (1+ num))
 ; Ŀ
 ;   Make the list back into a string.                                     
 ; 
         (setq gnustr "")
         (while (setq sub (car linn))
                (setq linn (cdr linn))
                (while (< (strlen sub) 9)
                       (setq sub (strcat sub " ")))
                (if (/= gnustr "")
                    (setq gnustr (strcat gnustr "  |  " sub))
                    (setq gnustr sub)))
 ; Ŀ
 ;   Add the new string to the string list.                                
 ; 
         (setq malist (cons gnustr malist)))
 ; Ŀ
 ;   Write the list back to the file.                                      
 ; 
  (setq malist (reverse malist))
  (setq num 0)
  (setq pref (getvar "dwgprefix"))
 ; Ŀ
 ;   Read the data file into a list.                                       
 ;   Don't assume there isn't one - after the first dwg. there will be.    
 ; 
  (setq fnam (strcat pref "cables.txt"))
  (if (setq fn (open fnam "w"))
      (progn
           (while (setq sub (nth num malist))
                  (setq num (1+ num))
                  (write-line sub fn))
           (close fn)))
 (princ))
 ; Ŀ
 ;   Xwrite end.                                                           
 ; 


 ; Ŀ
 ;   Everything below here is a c: routine.                                
 ; 


 ; Ŀ
 ;   Idf - find mismatches between entities and the database file but      
 ;   don't fix them.                                                       
 ; 
 (DEFUN C:IDF (/ skulok fnam cfglst apnam rad ss num enam entt typp vala
                                                                    valap ndxi)
 ; Ŀ
 ;   Set a few variables.                                                  
 ; 
  (setq apnam "cabx")
  (setq rad (/ (getvar "viewsize") 65))
 ; Ŀ
 ;   Try to load the grskull maker.                                        
 ; 
  (setq skulok (load "sk" ()))
 ; Ŀ
 ;   Get the current drawing path.                                         
 ; 
  (setq pref (getvar "dwgprefix"))
 ; Ŀ
 ;   Read the data file into a list.                                       
 ; 
  (setq fnam (findfile (strcat pref "cables.txt")))
  (setq cfglst (getcab fnam))
 ; Ŀ
 ;   Get an ss of all entities marked with the Cabx brand.                 
 ; 
  (if (setq ss (ssget "X" (list (list -3 (list apnam)))))
      (progn
           (setq num 0)
 ; Ŀ
 ;   Step through them.                                                    
 ; 
           (while (setq enam (ssname ss num))
                  (setq num (1+ num))
                  (setq entt (entget enam))
                  (setq typp (cdr (assoc 0 entt)))
 ; Ŀ
 ;   If the current entity is text then see if it needs to be updated.     
 ; 
                  (cond ((= typp "TEXT")
                         (setq pa (cdr (assoc 10 entt)))
                         (setq vala (cdr (assoc 1 entt)))
                         (setq ndxi (car (getxd enam apnam)))
                         (setq valap (cadr (assoc ndxi cfglst)))
                         (if (not (equal vala valap))
                             (if skulok
                                 (sk pa 0.075)
                                 (mark pa rad 1))))
 ; Ŀ
 ;   If it's a block step through the attributes and do the same.          
 ; 
                        ((= typp "INSERT")
                         (while (/= "SEQEND" (cdr (assoc 0 (setq entt
                                       (entget (setq enam (entnext enam)))))))
                                (setq pa (cdr (assoc 10 entt)))
                                (setq vala (cdr (assoc 1 entt)))
                                (setq ndxi (car (getxd enam apnam)))
                                (setq valap (cadr (assoc ndxi cfglst)))
                                (if (not (equal vala valap))
                                    (if skulok
                                        (sk pa 0.075)
                                        (mark pa rad 1)))))))))
 (princ))
 ; Ŀ
 ;   Idf end.                                                              
 ; 

 ; Ŀ
 ;   Igx - find all blocks with xdata, mark similar ones without.          
 ;   Calls Cstar.                                                          
 ; 
 (DEFUN C:IGX (/ apnam rad rota ss num enam entt blnam subnum)
 ; Ŀ
 ;   Set a few variables.                                                  
 ; 
  (setq apnam "cabx")
  (setq rad (/ (getvar "viewsize") 65))
  (setq rota 1)
 ; Ŀ
 ;   Get an ss of all blocks with Cabx xdata.                              
 ; 
  (if (setq ss (ssget "X" (list (cons 0 "insert") (list -3 (list apnam)))))
      (progn
           (setq num 0)
 ; Ŀ
 ;   Step through them, save their names to a list.                        
 ; 
           (while (setq enam (ssname ss num))
                  (setq num (1+ num))
                  (setq blnam (cdr (assoc 2 (entget enam))))
                  (if (not (member blnam blist))
                      (setq blist (cons blnam blist))))))
 ; Ŀ
 ;   Mark all insertions which don't have xdata.                           
 ; 
  (setq num 0)
  (while (setq blnam (nth num blist))
         (setq num (1+ num))
         (if (setq ss (ssget "x" (list (cons 2 blnam)
                                       (cons -4 "<not") (list -3 (list apnam))
                                       (cons -4 "not>"))))
             (progn
                  (setq subnum 0)
                  (while (setq enam (ssname ss subnum))
                         (setq subnum (1+ subnum))
                         (setq pa (cdr (assoc 10 (entget enam))))
                         (cstar pa rad rota 2)
                         (cstar pa (* rad 0.75) rota 1)
                         (cstar pa (* rad 0.5) rota 4)))))
 (princ))
 ; Ŀ
 ;   Igx end.                                                              
 ; 

 ; Ŀ
 ;   Indx - change an xdata index tag directly.                            
 ; 
 (DEFUN C:INDX (/ fnam apnam cfglst enam xdat tagp typp tagg ndxi)
  (setvar "cmdecho" 0)
  (setq apnam "cabx")
 ; Ŀ
 ;   Get the current drawing path.                                         
 ; 
  (setq pref (getvar "dwgprefix"))
 ; Ŀ
 ;   Read the data file into a list.                                       
 ; 
  (setq fnam (findfile (strcat pref "cables.txt")))
  (setq cfglst (getcab fnam))
 ; Ŀ
 ;   Get an entity or subentity.                                           
 ; 
  (setq enam (car (nentsel "Entity: ")))
 ; Ŀ
 ;   Warn if the entity already has a tag in xdata.  The user may be       
 ;   trying to change the value instead of the index tag.                  
 ; 
  (if (setq xdat (getxd enam apnam))
      (progn
           (setq tagp (cadr (assoc (car xdat) cfglst)))
           (prompt (strcat "\n*** Caution: That entity already has a tag: "
                           (car xdat) "/" tagp ". ***"))))
 ; Ŀ
 ;   Get the entity type, only change text and attributes.                 
 ; 
  (setq typp (cdr (assoc 0 (entget enam))))
  (if (not (member typp '("TEXT" "ATTRIB")))
      (write-line "That Wasn't Either Text Or An Attribute.")
      (progn
           (setq ndxi (getstring "\nIndex String: "))
 ; Ŀ
 ;   If the string is in the database then add the correct index tag to    
 ;   the entity as xdata and the string to the entity.                     
 ;   Note that we are doing a search by second element in the assoc list.  
 ; 
           (if (setq tagg (car (assoc ndxi cfglst)))
               (progn
                    (adxd enam (list "{" ndxi "1" "}") apnam)
                    (valp tagg enam))
 ; Ŀ
 ;   If the index tag wasn't in the database then fail - it must be        
 ;   added manually.  Otherwise would have to add it in aphabetical order  
 ;   and also add it to the index source file so that the attempt to       
 ;   find the one after it in the source file didn't fail.                 
 ;   Given the low probability of this being used it probably isn't        
 ;   worth the trouble.                                                    
 ; 
               (prompt
              "\n*** Operation failed - no such index tag in database. ***"))))
 (princ))

 ; Ŀ
 ;   Idt - find strings which should be tagged but aren't.                 
 ;   Calls Ci and Cstar.                                                   
 ;   Returns nothing.                                                      
 ; 
 (DEFUN C:IDT (/ apnam rad rota ss num enam entt typp pa stra sub)
  (setvar "cmdecho" 0)
  (setq apnam "cabx")
  (setq rad (/ (getvar "viewsize") 65))
  (setq rota (/ pi 3))
 ; Ŀ
 ;   Turn off group selection - otherwise entities may be selected >1x.    
 ; 
  (setq picks (getvar "pickstyle"))
  (setvar "pickstyle" 0)
 ; Ŀ
 ;   Something is bound to go wrong.                                       
 ; 
  (defun *error* (shk)
   (setvar "pickstyle" picks)
   (if shk (print shk))
  (princ))
 ; Ŀ
 ;   Get the current drawing path.                                         
 ; 
  (setq pref (getvar "dwgprefix"))
 ; Ŀ
 ;   Read the data file into a list.                                       
 ; 
  (setq fnam (findfile (strcat pref "cables.txt")))
  (setq cfglst (getcab fnam))
 ; Ŀ
 ;   Text.                                                                 
 ; 
  (setq ss (ssget "X" (list (cons 0 "TEXT"))))
  (setq num 0)
  (while (and ss (setq enam (ssname ss num)))
         (setq num (1+ num))
         (setq entt (entget enam (list apnam)))
         (setq stra (cdr (assoc 1 entt)))
 ; Ŀ
 ;   See if the string is in the database.                                 
 ; 
         (setq sub (a22oc stra cfglst 1))
         (if (and sub (not (assoc -3 entt)))
             (progn
                  (setq pa (spit entt))
                  (cstar pa rad rota 7)
                  (setq rota (+ rota 0.1)))))
 ; Ŀ
 ;   Inserts.                                                              
 ; 
  (setq num 0)
  (setq ss (ssget "X" (list (cons 66 1) (cons 0 "INSERT"))))
  (while (and ss (setq enam (ssname ss num)))
         (setq num (1+ num))
         (while (/= "SEQEND" (cdr (assoc 0 (setq entt (entget
                                  (setq enam (entnext enam)) (list apnam))))))
                (setq stra (cdr (assoc 1 entt)))
                (setq sub (a22oc stra cfglst 1))
                (if (and sub (not (assoc -3 entt)))
                    (progn
                         (setq pa (spit entt))
                         (cstar pa rad rota 7)
                         (setq rota (+ rota 0.1))))))
 ; Ŀ
 ;   Clean up and end.                                                     
 ; 
  (*error* ())
 (princ))
 ; Ŀ
 ;   Idt end.                                                              
 ; 

 ; Ŀ
 ;   Idux - identify all un-tagged entities.                               
 ;   Calls Ci and Cstar.                                                   
 ;   Returns nothing.                                                      
 ; 
 (DEFUN C:IDUX (/ apnam rad rota ss num enam entt typp pa)
  (setvar "cmdecho" 0)
  (setq apnam "cabx")
  (setq rad (/ (getvar "viewsize") 65))
  (setq rota (/ pi 3))
 ; Ŀ
 ;   This turns off group selection - otherwise entities in groups may     
 ;   be changed twice with bad things happening as a result.               
 ; 
  (setq picks (getvar "pickstyle"))
  (setvar "pickstyle" 0)
 ; Ŀ
 ;   Something is bound to go wrong.                                       
 ; 
  (defun *error* (shk)
   (setvar "pickstyle" picks)
   (if shk (print shk))
  (princ))
 ; Ŀ
 ;   Text.                                                                 
 ; 
  (setq ss (ssget "X" (list (cons 0 "TEXT"))))
  (setq num 0)
  (while (and ss (setq enam (ssname ss num)))
         (setq entt (entget enam (list apnam)))
         (setq num (1+ num))
         (if (not (assoc -3 entt))
             (progn
                  (setq pa (spit entt))
                  (cstar pa rad rota 8)
                  (cstar pa (* rad 0.8) rota 8)
                  (cstar pa (* rad 0.6) rota 2)
                  (cstar pa (* rad 0.4) rota 2)
                  (setq rota (+ rota 0.1)))))
 ; Ŀ
 ;   Inserts.                                                              
 ; 
  (setq num 0)
  (setq ss (ssget "X" (list (cons 66 1) (cons 0 "INSERT"))))
  (while (and ss (setq enam (ssname ss num)))
         (setq num (1+ num))
         (while (/= "SEQEND" (cdr (assoc 0 (setq entt (entget
                                  (setq enam (entnext enam)) (list apnam))))))
                (if (not (assoc -3 entt))
                    (progn
                         (setq pa (spit entt))
                         (cstar pa rad rota 8)
                         (cstar pa (* rad 0.8) rota 8)
                         (cstar pa (* rad 0.6) rota 1)
                         (cstar pa (* rad 0.4) rota 1)
                         (setq rota (+ rota 0.1))))))
 ; Ŀ
 ;   Clean up and end.                                                     
 ; 
  (*error* ())
 (princ))
 ; Ŀ
 ;   Idux end.                                                             
 ; 

 ; Ŀ
 ;   Idx - identify all tagged entities.                                   
 ;   Calls Ci and Cstar.                                                   
 ;   Returns nothing.                                                      
 ; 
 (DEFUN C:IDX (/ apnam rad rota ss num enam entt typp pa)
  (setvar "cmdecho" 0)
  (setq apnam "cabx")
  (setq rad (/ (getvar "viewsize") 65))
  (setq rota (/ pi 3))
 ; Ŀ
 ;   Get an ss of all entities with Cabx xdata.                            
 ; 
  (if (setq ss (ssget "X" (list (list -3 (list apnam)))))
      (progn
           (setq num 0)
 ; Ŀ
 ;   Step through them.                                                    
 ; 
           (while (setq enam (ssname ss num))
                  (setq num (1+ num))
                  (setq entt (entget enam))
                  (setq typp (cdr (assoc 0 entt)))
 ; Ŀ
 ;   If the current entity is text then mark and update it.                
 ; 
                  (cond ((= typp "TEXT")
                         (setq pa (spit entt))
                         (ci pa rad 3)
                         (cstar pa rad rota 8)
                         (setq rota (+ rota 0.1)))
 ; Ŀ
 ;   If it's a block step through the attributes and do the same.          
 ; 
                        ((= typp "INSERT")
                         (setq pa (cdr (assoc 10 entt)))
                         (ci pa (* rad 0.65) 2)
                         (cstar pa (* rad 0.65) rota 8)
                         (setq rota (+ rota 0.1))
                         (while (/= "SEQEND" (cdr (assoc 0 (setq entt (entget
                                   (setq enam (entnext enam)) (list apnam))))))
                                (if (assoc -3 entt)
                                    (progn
                                         (setq pa (spit entt))
                                         (ci pa rad 1)
                                         (cstar pa rad rota 8)
                                         (setq rota (+ rota 0.1))))))))
 ; Ŀ
 ;   Mention how many entities were marked ... so far en masse.            
 ; 
           (write-line (strcat "\nEntities with Cabx tags: " (itoa num))))
 ; Ŀ
 ;   Or point out that there were none.                                    
 ; 
      (write-line "\nNo Cabx Entities found."))
 (princ))
 ; Ŀ
 ;   Idx end.                                                              
 ; 

 ; Ŀ
 ;   Showx - get the index tag from the xdata from an entity.              
 ;  This needs to be fixed to work with tagged blocks - parent entities.
 ; 
 (DEFUN C:SHOWX (/ apnam enam enttx ndata)
  (setq apnam "cabx")
  (if (and (setq enam (car (nentsel "Entity: ")))
           (setq entt (entget enam (list apnam)))
           (member (cdr (assoc 0 entt)) '("TEXT" "ATTRIB"))
           (assoc -3 entt))
      (progn
           (setq ndata (lizx entt))
           (prompt (strcat "\nIndex Tag: " (car ndata))))
      (prompt "\nThat wasn't text or an attdef with xdata."))
 (princ))
 ; Ŀ
 ;   Showx end.                                                            
 ; 

 ; Ŀ
 ;   Sltx - copy a text file, ditching any words which are too long.       
 ;   Brooks no arguments, returns nothing.                                 
 ; 
 (DEFUN C:SLTX (/ num subnum fn fo str)
  (setq num 0)
  (setq subnum 0)
  (if (and (setq fn (open "corncob.txt" "r"))
           (setq fo (open "Bocc4.txt" "w")))
      (while (setq str (read-line fn))
             (setq num (1+ num))
             (if (< (strlen str) 5)
                 (progn
                      (setq subnum (1+ subnum))
                      (write-line str fo)))))
  (if fn (close fn))
  (if fo (close fo))
  (write-line (strcat (itoa subnum) "/" (itoa num)))
 (princ))
 ; Ŀ
 ;   C:Sltx end.                                                           
 ; 

 ; Ŀ
 ;   Sux - Suck xdata and a value from another entity.                     
 ; 
 (DEFUN C:SUX (/ apnam enam xdat tagg typp ndxi nunam)
  (setq apnam "cabx")
 ; Ŀ
 ;   Get an entity or subentity, get its xdata.                            
 ; 
  (setq enam (car (nentsel "Source Entity: ")))
  (setq xdat (getxd enam apnam))
  (setq tagg (cdr (assoc 1 (setq entt (entget enam)))))
 ; Ŀ
 ;   Get the entity type, only work with text and attributes.              
 ; 
  (setq typp (cdr (assoc 0 entt)))
  (cond ((not (member typp '("TEXT" "ATTRIB")))
         (prompt "\n*** That wasn't text."))
        ((null xdat)
         (prompt "\n*** No Index Data Found."))
 ; Ŀ
 ;   We are okay to proceed.  Copy the data into new entities as long as   
 ;   the user is willing to keep selecting them.                           
 ; 
        (t
         (setq ndxi (car xdat))
         (while (setq nunam (car (nentsel (strcat
                        "\nNew Entity For Index/Value <" ndxi "/" tagg ">:"))))
 ; Ŀ
 ;   Add the index tag to the entity as xdata and install the string.      
 ;   Note that we aren't checking the database - the xdata may be crap.    
 ;   (And the user might want that.)                                       
 ; 
                (setq typp (cdr (assoc 0 (entget nunam))))
                (cond ((not (member typp '("TEXT" "ATTRIB")))
                       (prompt "\n*** That Wasn't Text Or An Attribute."))
                      (t
                       (adxd nunam (list "{" ndxi "1" "}") apnam)
                       (valp tagg nunam))))))
 (princ))

 ; Ŀ
 ;   Upcx - update any entities with Cabx xdata from a file.               
 ; 
 (DEFUN C:UPCX (/ *error* cfglst apnam rad rota ss num enam entt typp esav
                                                                bltyp updx)
  (setvar "cmdecho" 0)
  (command "undo" "be")
 ; Ŀ
 ;   Set a few variables.                                                  
 ; 
  (setq apnam "cabx")
  (setq rad (/ (getvar "viewsize") 65))
  (setq rota 1)
 ; Ŀ
 ;   Make a local error handler.                                           
 ; 
  (defun *error* (shk)
   (if shk (print shk))
  (princ))
 ; Ŀ
 ;   Get the current drawing path.                                         
 ; 
  (setq pref (getvar "dwgprefix"))
 ; Ŀ
 ;   Read the data file into a list.                                       
 ; 
  (setq fnam (findfile (strcat pref "cables.txt")))
  (setq cfglst (getcab fnam))
 ; Ŀ
 ;   Get an ss of all entities marked with the Cabx brand.                 
 ; 
  (if (setq ss (ssget "X" (list (list -3 (list apnam)))))
      (progn
           (setq num 0)
 ; Ŀ
 ;   Step through them.                                                    
 ; 
           (while (setq enam (ssname ss num))
                  (setq num (1+ num))
                  (setq entt (entget enam))
                  (setq typp (cdr (assoc 0 entt)))
 ; Ŀ
 ;   Get the xdata for each one so we can decide how to update blocks.     
 ;   Xtype returns a string: "none" = no xdata was found, "atts" = each    
 ;   attribute will be tagged, do individually, "dash" = get the string    
 ;   from the database, split at dashes and apply to attributes in order.  
 ; 
                  (setq bltyp (xtype enam apnam))
 ; Ŀ
 ;   If the current entity is text then mark and update it.                
 ; 
                  (cond ((= typp "TEXT")
                         (upex enam cfglst apnam rad rota)
                         (setq rota (+ rota 0.2)))
 ; Ŀ
 ;   If it's a block either step through the attributes and apply values   
 ;   from the database individually, or get a string and call Binx to      
 ;   split the string at dashes and put the subs into the atts in order.   
 ; 
                        ((and (= typp "INSERT") (= bltyp "atts"))
                         (setq esav enam)
                         (setq updx nil)
                         (while (/= "SEQEND" (cdr (assoc 0 (entget
                                                (setq enam (entnext enam))))))
                                (setq updxp (upex enam cfglst apnam rad rota))
                                (setq updx (or updx updxp))
                                (setq rota (+ rota 0.2)))
                         (if updx (entupd esav)))
                        ((and (= typp "INSERT") (= bltyp "dash"))
                         (setq esav enam)
                         (updsh enam cfglst apnam rad rota)
                         (setq rota (+ rota 0.2))
                         (entupd esav))))))
 (princ))
 ; Ŀ
 ;   Upcx end.                                                             
 ; 

 ; Ŀ
 ;   This is similar to Cabx (which adds a value and makes sure it has     
 ;   an index tag in xdata and in the database) but Upx changes the        
 ;   string value associated with the index tag in the database.           
 ;   If there is no xdata index string attached to the entity then Upx     
 ;   adds one exactly as Cabx does.                                        
 ; 
 (DEFUN C:UPX (/ apnam fnam cfglst enam typp xdat datt tagp ndxi tagg)
  (setvar "cmdecho" 0)
  (setq apnam "cabx")
 ; Ŀ
 ;   Get the current drawing path.                                         
 ; 
  (setq pref (getvar "dwgprefix"))
 ; Ŀ
 ;   Read the data file into a list.                                       
 ; 
  (setq fnam (findfile (strcat pref "cables.txt")))
  (setq cfglst (getcab fnam))
 ; Ŀ
 ;   Get an entity or subentity.                                           
 ; 
  (setq enam (car (nentsel "Entity: ")))
 ; Ŀ
 ;   Get the entity type.                                                  
 ; 
  (setq typp (cdr (assoc 0 (entget enam))))
 ; Ŀ
 ;   See if the entity has xdata and if there is a matching line in the    
 ;   configuration file.                                                   
 ; 
  (if (setq xdat (getxd enam apnam))
      (setq datt (assoc (car xdat) cfglst)))
 ; Ŀ
 ;   1. Only work with text and attributes.                                
 ; 
  (cond ((not (member typp '("TEXT" "ATTRIB")))
         (write-line "That Wasn't Either Text Or An Attribute."))
 ; Ŀ
 ;   2. If the entity doesn't have a tag in xdata ... quit.                
 ; 
        ((null xdat)
         (prompt "\n*** No Database Index Tag.  Use Cabx to add one."))
 ; Ŀ
 ;   3. If the entity has an index tag but it isn't in the database, quit. 
 ; 
        ((and xdat (null datt))
         (prompt (strcat "\n*** Error: No Data Associated With Index Tag: "
                         (car xdat))))
 ; Ŀ
 ;   4. The entity already has a tag in xdata.                             
 ;      Warn the user but continue.                                        
 ; 
        ((and xdat (setq tagp (cadr datt)))
         (setq ndxi (car xdat))
         (prompt (strcat "\n*** Caution: This will change the string"
                         " for the index tag " ndxi "."))
         (setq tagg (getstring
                     "\nNew Displayed Value (Will Replace All Instances): "))
 ; Ŀ
 ;   Update the value in the database.                                     
 ; 
         (dbrep ndxi tagg fnam)
 ; Ŀ
 ;   Replace the displayed string value in the entity.                     
 ; 
         (valp tagg enam)))
 (princ))

 ; Ŀ
 ;   Wrx - The drawing may be edited with a regular text editor - tt or    
 ;   deep.  Write these changes back to the dbase file.                    
 ;   In other words commit selected entities or all in the drawing.        
 ; 
 (DEFUN C:WRX (/ apnam rad *error* fnam cfglst ss num enam entt typp vall xdat
                                                 ndxi datt tagg dat1 pa esav)
  (setvar "cmdecho" 0)
 ; Ŀ
 ;   Set a few variables.                                                  
 ; 
  (setq apnam "cabx")
  (setq rad (/ (getvar "viewsize") 65))
 ; Ŀ
 ;   Make a local error handler.                                           
 ; 
  (defun *error* (shk)
   (if shk (print shk))
  (princ))
 ; Ŀ
 ;   Get the current drawing path.                                         
 ; 
  (setq pref (getvar "dwgprefix"))
 ; Ŀ
 ;   Read the data file into a list.                                       
 ; 
  (setq fnam (findfile (strcat pref "cables.txt")))
  (setq cfglst (getcab fnam))
 ; Ŀ
 ;   Get an ss of all entities marked with the Cabx brand.                 
 ; 
  (if (setq ss (ssget "X" (list (list -3 (list apnam)))))
      (progn
           (setq num 0)
 ; Ŀ
 ;   Step through them.                                                    
 ; 
           (while (setq enam (ssname ss num))
                  (setq num (1+ num))
                  (setq entt (entget enam))
                  (setq typp (cdr (assoc 0 entt)))
 ; Ŀ
 ;   If the current entity is text then see if the value matches the       
 ;   index tag, if not then update the index tag in the database file.     
 ; 
                  (cond ((= typp "TEXT")
                         (setq vall (cdr (assoc 1 entt)))
 ; Ŀ
 ;   Get the index tag from the xdata attached to the entity and see if    
 ;   the displayed value matches the value in the database file for that   
 ;   index tag.                                                            
 ; 
                         (if (and (setq xdat (getxd enam apnam))
                                  (setq ndxi (car xdat))
                                  (setq datt (assoc ndxi cfglst))
                                  (setq tagg (cadr datt))
                                  (not (equal tagg vall)))
 ; Ŀ
 ;   Update the value in the database.                                     
 ; 
                             (progn
                                  (dbrep ndxi vall fnam)
                                  (setq dat1 (subst vall tagg datt))
                                  (setq cfglst (subst dat1 datt cfglst))
                                  (setq pa (spit entt))
                                  (olaf pa rad 0.1 0 140 9))))
 ; Ŀ
 ;   If it's a block step through the attributes and do the same.          
 ; 
                        ((= typp "INSERT")
                         (setq esav enam)
                         (while (/= "SEQEND" (cdr (assoc 0 (setq entt (entget
                                                (setq enam (entnext enam)))))))
                                (setq vall (cdr (assoc 1 entt)))
                                (if (and (setq xdat (getxd enam apnam))
                                         (setq ndxi (car xdat))
                                         (setq datt (assoc ndxi cfglst))
                                         (setq tagg (cadr datt))
                                         (not (equal tagg vall)))
 ; Ŀ
 ;   Update the value in the database.                                     
 ; 
                                    (progn
                                         (dbrep ndxi vall fnam)
                                         (setq dat1 (subst vall tagg datt))
                                         (setq cfglst (subst dat1 datt cfglst))
                                         (setq pa (spit entt))
                                         (olaf pa rad 0.1 0 140 9)))))))))
 (princ))
 ; Ŀ
 ;   Wrx end.                                                              
 ; 

 ; Ŀ
 ;   Xcab - Make a database file from a set of drawings if the original    
 ;   is missing.                                                            
 ; 
 (DEFUN C:XCAB (/ *error* liss apnam rad rota ss num enam entt typp esav
                                                                    updx)
 ; Ŀ
 ;   Set a few variables.                                                  
 ; 
  (setq apnam "cabx")
  (setq rad (/ (getvar "viewsize") 265))
  (setq rota 1)
 ; Ŀ
 ;   Make a local error handler.                                           
 ; 
  (defun *error* (shk)
   (if shk (print shk))
  (princ))
 ; Ŀ
 ;   Get the current drawing path.                                         
 ; 
  (setq pref (getvar "dwgprefix"))
 ; Ŀ
 ;   Read the data file into a list.                                       
 ;   Don't assume there isn't one - after the first dwg. there will be.    
 ; 
  (if (setq fnam (findfile (strcat pref "cables.txt")))
      (setq liss (getcab fnam)))
 ; Ŀ
 ;   Get an ss of all entities marked with the Cabx brand.                 
 ; 
  (if (setq ss (ssget "X" (list (list -3 (list apnam)))))
      (progn
           (setq num 0)
 ; Ŀ
 ;   Step through them.                                                    
 ; 
           (while (setq enam (ssname ss num))
                  (setq num (1+ num))
                  (setq entt (entget enam))
                  (setq typp (cdr (assoc 0 entt)))
 ; Ŀ
 ;   If the current entity is text then extract its data and update it     
 ;   in - or add it to - cfglst.                                           
 ; 
                  (cond ((= typp "TEXT")
                         (setq pa (spit entt))
                         (cvo pa rad 4)
                         (mark pa (* rad 12) 210)
                         (setq xdat (getxd enam "cabx"))    ; xdata
                         (setq vala (cdr (assoc 1 entt)))   ; string
                         (setq liss (upcfg liss xdat vala)))
 ; Ŀ
 ;   If it's a block step through the attributes and do the same.          
 ; 
                        ((= typp "INSERT")
                         (setq esav enam)
                         (while (/= "SEQEND" (cdr (assoc 0 (setq entt (entget
                                                (setq enam (entnext enam)))))))
                                (setq pa (spit entt))
                                (cvo pa rad 4)
                                (mark pa (* rad 12) 220)
                                (setq xdat (getxd enam "cabx"))    ; xdata
                                (setq vala (cdr (assoc 1 entt)))   ; string
                                (setq liss (upcfg liss xdat vala))))))
 ; Ŀ
 ;   Now write the list back to the file.                                  
 ;   Modify the reader to include and tag comments so they aren't lost.    
 ; 
           (setq cfglst (horizf liss))
           (xwrite liss fnam)))
 (princ))
 ; Ŀ
 ;   Xcab end.                                                             
 ; 

 ; Ŀ
 ;   Cabx.                                                                 
 ; 
 (DEFUN C:CABX (/ fnam apnam cfglst enam entt xdat tagp typp tagg ndxi gnundx)
  (setvar "cmdecho" 0)
  (setq apnam "cabx")
 ; Ŀ
 ;   Get the current drawing path.                                         
 ; 
  (setq pref (getvar "dwgprefix"))
 ; Ŀ
 ;   Read the data file into a list.                                       
 ; 
  (setq fnam (findfile (strcat pref "cables.txt")))
  (setq cfglst (getcab fnam))
 ; Ŀ
 ;   Get an entity or subentity.                                           
 ; 
  (setq enam (car (nentsel "Entity: ")))
 ; Ŀ
 ;   Warn if the entity already has a tag in xdata.  The user may be       
 ;   trying to change the value instead of the index tag.                  
 ; 
  (if (setq xdat (getxd enam apnam))
      (progn
           (setq tagp (cadr (assoc (car xdat) cfglst)))
           (prompt (strcat "\n*** Caution: That entity already has a tag: "
                           (car xdat) "/" tagp ". ***"))
 ; Ŀ
 ;   If the grskull maker can be loaded add a grdraw warning here.         
 ; 
           (if (setq skulok (load "sk" ()))
               (progn
                    (setq pa (spit (entget enam)))
                    (sk pa 0.1)))))
 ; Ŀ
 ;   Get the entity type, only work with text and attributes.              
 ; 
  (setq typp (cdr (assoc 0 (setq entt (entget enam)))))
  (if (not (member typp '("TEXT" "ATTRIB")))
 ; Ŀ
 ;   ** Should check for being encapsulated in a block or xref.            
 ;   ** Need a Get Parent routine.                                         
 ; 
      (write-line "That Wasn't Either Text Or An Attribute.")
      (progn
           (setq stra (cdr (assoc 1 entt)))
           (setq tagg (getstring (strcat "\nTag <" stra ">: ")))
           (if (= tagg "") (setq tagg stra))
 ; Ŀ
 ;   If the string is in the database then add the correct index tag to    
 ;   the entity as xdata and the string to the entity.                     
 ;   Note that we are doing a search by second element in the assoc list.  
 ; 
           (if (and cfglst (setq ndxi (caar (a22oc tagg cfglst 1))))
               (progn
                    (adxd enam (list "{" ndxi "1" "}") apnam)
                    (valp tagg enam))
 ; Ŀ
 ;   If the string is not in the database then call Getbac to get a new    
 ;   index tag, Addcb to add it and the string to the database file, Axdx  
 ;   to add the index tag to the entity as xdata, and Valp to put the      
 ;   string into the entity.                                               
 ; 
               (progn
                    (setq gnundx (getbac (findfile "index.idx") (caar cfglst)))
                    (addcb gnundx tagg fnam)
                    (adxd enam (list "{" gnundx "1" "}") apnam)
                    (valp tagg enam)))))
 (princ))

 ; Ŀ
 ;   Xdash - apply an xdata tag to a block.  This will tell Upcx to split  
 ;   the string from the dbase file at dashes and apply it to attributes   
 ;   in sequence.                                                          
 ; 
 (DEFUN C:XDASH (/ enam)
  (setvar "cmdecho" 0)
  (setq enam (car (entsel "Multi-Attribute Block: ")))
  (xdash enam nil "cabx")
 (princ))

 ; Ŀ
 ;   Xxdash - apply an xdata tag to multiple blocks, splitting the         
 ;   string at dashes and applying it to attributes in sequence.           
 ;   The existing strings are taken to be the desired ones.                
 ; 
 (DEFUN C:XXDASH (/ rad enam ss num enam)
  (setvar "cmdecho" 0)
  (setq rad (/ (getvar "viewsize") 265))
  (prompt "Multi-Attribute Blocks To Tag: ")
  (setq ss (ssget (list (cons 66 1) (cons 0 "INSERT"))))
  (setq num 0)
  (while (and ss (setq enam (ssname ss num)))
         (setq num (1+ num))
         (cvo (cdr (assoc 10 (entget enam))) rad 4)
         (xdash enam t "cabx"))
 (princ))

 ; Ŀ
 ;   Xxall - Write selected entities to the database, assuming the         
 ;   current string is correct, add xdata tags.                            
 ;   This is for making an existing drawing into a linked one.             
 ;   If tagged text is encountered, ditch the tags.                        
 ;   (Basically in the interest of getting this done, but the user can     
 ;   run Idx to make sure he's not overwriting anything.                   
 ;   Read in the database file, if any, and tag any strings which have     
 ;   matches in the database.                                              
 ;   Get a list of the selected strings, alphabetize.                      
 ;   Add to the database file, then add tags to the drawing.               
 ;   This can be used to do one tag or many, although it won't be able to  
 ;   do single picks.                                                      
 ;   It also doesn't ignore invisible attributes - this is a first shot.   
 ; 
 (DEFUN C:XXALL (/ apnam pref fnamp fnam cfglst ss strlis num stra gnundx
                                                            idxnam gnulst)
  (setvar "cmdecho" 0)
  (setq apnam "cabx")
 ; Ŀ
 ;   Make a local error handler.                                           
 ; 
  (defun *error* (shk)
   (if (and shk (/= shk "quit / exit abort"))
       (print shk))
  (princ))
 ; Ŀ
 ;   Get the current drawing path.                                         
 ; 
  (setq pref (getvar "dwgprefix"))
 ; Ŀ
 ;   Read the data file into a list.                                       
 ;   This will be nil if it can't be found.                                
 ; 
  (if (setq fnamp (findfile (setq fnam (strcat pref "cables.txt"))))
      (setq fnam fnamp))
  (setq cfglst (getcab fnam))
 ; Ŀ
 ;   Get an ss of entities.                                                
 ; 
  (write-line "Select entities to link to Cables.txt: ")
  (if (setq ss (ssget (list (cons 0 "text,insert"))))
      (progn
 ; Ŀ
 ;   Get all unique text strings therein.                                  
 ;   Actually the common use of unique in programming is incorrect -       
 ;   we're getting all strings and ditching duplicates, not ditching       
 ;   strings which have duplicates.                                        
 ; 
           (setq strlis (getals ss))
 ; Ŀ
 ;   Update Cfglst with all strings it doesn't already contain, get new    
 ;   database tags for them.                                               
 ;   Keep the new ones in an additional separate list to avoid having to   
 ;   decide which parts of the file to preserve.                           
 ; 
           (setq num 0)
           (while (setq stra (nth num strlis))
                  (setq num (1+ num))
                  (if (not (a22oc stra cfglst 1))
                      (progn
                           (if (not (setq idxnam (findfile "index.idx")))
                               (progn
                                    (prompt "\n** Abort: Missing Index.idx **")
                                    (exit))
                               (setq gnundx (getbac idxnam (caar cfglst))))
                           (setq cfglst (cons (list gnundx stra) cfglst))
                           (setq gnulst (cons (list gnundx stra) gnulst)))))
 ; Ŀ
 ;   Write the strings from Gnulst to the database file.                   
 ; 
           (if gnulst
               (admult (reverse gnulst) fnam)
               (prompt "\n** Warning: No Data Added To Cables.txt. **"))
 ; Ŀ
 ;   Step through ss and call Axdx to add the index tag to each entity     
 ;   as xdata.  Leave the strings untouched.                               
 ;   Note that we are doing a search by second element in the assoc list.  
 ; 
           (pumul ss cfglst apnam)))
 (princ))


(princ)

