 ; Ŀ
 ;   Malaya - see if a layer exists and has the right colour and linetype. 
 ;   If not then take steps...                                             
 ;   Copyright 1999 - 2018 by Rocket Software Ltd.                         
 ;   If a layer exists but has the wrong colour or linetype, fix it.       
 ;   If it doesn't exist, make it, unless there is no layer data,          
 ;   in which case make it with the default settings.                      
 ;   Once it exists, make it current.                                      
 ;   This avoids having to insert the whole set of layers to get one.      
 ;                                                                         
 ;   Also contains utilities:                                              
 ;   Coid - Make all layers named in an internal list.                     
 ;   Llout - extract layer data into a file in the format Malaya likes.    
 ;   Fixla - make all layers in a drawing correct (if Malaya has data).    
 ;   Lap - make the layer of a picked entity correct (if Malaya has data). 
 ;   Xlay - set up xref layers properly.                                   
 ;                                                                         
 ;   Correct layer settings are contained in (prlist) which must be        
 ;   edited if new layers are added or existing ones changed.              
 ;   Also C:Coid and C:Xlay contain editable internal data lists.          
 ;                                                                         
 ;   Notes:                                                                
 ;                                                                         
 ;   Malaya First checks to see if the layer name it has been handed is a  
 ;   variable, if so it uses the name it contains, otherwise the string    
 ;   is taken to be a layer name.  In either case it is checked against    
 ;   an internal list.                                                     
 ;                                                                         
 ;   This being the case, the standard layer names can be placed directly  
 ;   in the menu file, and if different names are required they can be     
 ;   used merely by considering them to be variables and setting them to   
 ;   the new names. In other words, we can maintain client layer files in  
 ;   specific job subdirectories.  (Layer data must be present in the      
 ;   layer data list LaList in Malaya.)                                    
 ;                                                                         
 ; 

 ; Ŀ
 ;   Coid - utility - make all layers named in a list using the setting    
 ;   data from the list made by Prlist.                                    
 ;   This could be modified to read a data list from an external file      
 ;   and thus use one of several client-specific layer sets.               
 ;   Put the default file in c:\acad\support and the client one (if any)   
 ;   in the job-specific start directory.                                  
 ;   It could also run off the current profile name.                       
 ; 
 (DEFUN C:COID (/ maclst lanam laydat)
 ; Ŀ
 ;   It might be a good idea to update this list before using Coid.        
 ; 
  (setq maclst (list "building" "busbar" "cable"
                     "cableug" "center" "cond_cable" "contin"))
 ; Ŀ
 ;   Now step through the desired layer list and call malaya to make or    
 ;   fix each one.                                                         
 ; 
  (while (setq lanam (car maclst))
         (setq maclst (cdr maclst))
         (setq lanam (strcase lanam t))
         (malaya lanam))
 (princ))
 ; Ŀ
 ;   Coid end.                                                             
 ; 

 ; Ŀ
 ;   Fixla - utility - make sure the layer settings are correct for all    
 ;   known layers in the current drawing.                                  
 ;   Also lists all nonstandard layers.                                    
 ; 
 (DEFUN C:FIXLA (/ lalist rewind clay lanam)
 ; Ŀ
 ;   Call prlist to make the layer data list.                              
 ; 
  (setq lalist (prlist))
  (setq rewind T)
 ; Ŀ
 ;   Now step through the layer data table and fix each one as required.   
 ; 
  (while (setq clay (tblnext "layer" rewind))
         (setq rewind ())
         (setq lanam (cdr (assoc 2 clay)))
 ; Ŀ
 ;   See if there a a data sublist in lalist for the layer in question.    
 ;   If so call malaya to make or fix it.                                  
 ; 
         (setq lanam (strcase lanam t))
         (if (assoc lanam lalist)
             (malaya lanam)
 ; Ŀ
 ;   Otherwise print an error message, unless the layer is xref dependent. 
 ; 
             (if (/= 16 (logand 16 (cdr (assoc 70 clay))))
                 (prompt (strcat "\nNonstandard layer: " lanam)))))
 (princ))
 ; Ŀ
 ;   Fixla end.                                                            
 ; 

 ; Ŀ
 ;   Lap - utility - fix the layer settings for a picked layer.            
 ; 
 (DEFUN C:LAP (/ lalist enam lanam)
 ; Ŀ
 ;   Call prlist to make the layer data list.                              
 ; 
  (setq lalist (prlist))
 ; Ŀ
 ;   Get an entity, extract its layer name.                                
 ; 
  (if (setq enam (car (nentsel)))
      (progn
           (setq lanam (cdr (assoc 8 (entget enam))))
 ; Ŀ
 ;   See if there a data sublist in lalist for the layer in question.      
 ;   If so call malaya to make or fix it.                                  
 ; 
           (setq lanam (strcase lanam t))
           (cond ((assoc lanam lalist)
                  (malaya lanam)
                  (prompt (strcat "\nLayer " lanam " adjusted.")))
 ; Ŀ
 ;   Otherwise print an error message, unless the layer is xref dependent. 
 ; 
                 ((= 16 (logand 16 (cdr (assoc 70 (tblsearch "layer" lanam)))))
                  (prompt (strcat "\nCan't Change Xref Layer: " lanam)))
                 (t
                  (prompt (strcat "\nUnknown layer: " lanam))))))
 (princ))
 ; Ŀ
 ;   Lap end.                                                              
 ; 

 ; Ŀ
 ;   Subroutine Lava - see if a layer exists and has the right colour      
 ;   and linetype.                                                         
 ;   If it exists but has the wrong colour or linetype, fix it.            
 ;   If it doesn't exist then make it.                                     
 ;                                                                         
 ;   Takes one argument, Laydat, a list: (layer_name colour linetype)      
 ;   Returns nothing.                                                      
 ; 
 (DEFUN LAVA (laydat / lanam coll lintyp ladata)
 ; Ŀ
 ;   Extract the correct data from the list.                               
 ; 
  (setq lanam (car laydat))
  (setq coll (nth 1 laydat))
  (setq lintyp (nth 2 laydat))
 ; Ŀ
 ;   Check the linetype, load it if it isn't already present.              
 ;   (Assumes that the linetypes are in the file Acad.lin.)                
 ; 
  (if (null (tblsearch "ltype" lintyp))
      (command "linetype" "load" lintyp "acad" ""))
 ; Ŀ
 ;   Get the layer data from the layer table.                              
 ; 
  (setq ladata (tblsearch "layer" lanam))
 ; Ŀ
 ;   If the layer doesn't exist or the properties are wrong, make/fix it.  
 ; 
  (cond ((null ladata)
         (command "layer" "new" lanam
                          "colour" coll lanam
                          "ltype" lintyp lanam ""))
        ((or (/= (cdr (assoc 6 ladata)) (strcase lintyp))
             (/= (cdr (assoc 62 ladata)) coll))
         (command "layer" "set" lanam
                          "colour" coll lanam
                          "ltype" lintyp lanam ""))))
 ; Ŀ
 ;   Subroutine Lava end.                                                  
 ; 

 ; Ŀ
 ;   Llout - utility - write the name, colour, and linetype of each layer  
 ;   to a file in the format that Malaya likes.                            
 ;   You can now copy layer settings out of a file for use in malaya.      
 ;   Consider going with an external layer list...maybe not, since one     
 ;   loses the advantage of an interpreted language.  Also would be        
 ;   reading a separate file to make each layer.                           
 ;   Maybe a separate subroutine at the start of the file to make Lalist.  
 ;   The sublist format is:  '(Layername colournumber linetype)            
 ; 
 (DEFUN C:LLOUT (/ rewind fnam clay clam cltype ccolo)
  (setq fnam (open "layer.log" "w"))
  (setq rewind T)
  (while (setq clay (tblnext "layer" rewind))
         (setq rewind ())
         (setq clam (cdr (assoc 2 clay)))
         (setq cltype (cdr (assoc 6 clay)))
         (setq ccolo (cdr (assoc 62 clay)))
         (print (list clam ccolo cltype) fnam))
  (close fnam)
 (princ))
 ; Ŀ
 ;   Llout end.                                                            
 ; 

 ; Ŀ
 ;   Subroutine Malaya - the engine.                                       
 ;   Takes one argument, a layer name string.                              
 ; 
 (DEFUN MALAYA (lanam / regg exp lalist laydat coll lintyp ladata)
 ; Ŀ
 ;   Since this is intended to work with layer name variables as well as   
 ;   layer name strings eval the input so that the variable Lanam          
 ;   contains a string in either case.                                     
 ; 
  (if (= (type (setq lamp (eval (read lanam)))) 'STR)
      (setq lanam lamp))
 ; Ŀ
 ;   Save settings and reset.                                              
 ; 
  (setq regg (getvar "regenmode"))
  (setvar "regenmode" 0)
  (setq exp (getvar "expert"))
  (setvar "expert" 1)
 ; Ŀ
 ;   Call prlist to make the layer data list.                              
 ; 
  (setq lalist (prlist))
 ; Ŀ
 ;   See if there a a data sublist in lalist for the layer in question.    
 ;   If so call Lava to make or fix it.                                    
 ; 
  (setq lanam (strcase lanam t))
  (if (setq laydat (assoc lanam lalist))
      (lava laydat)
 ; Ŀ
 ;   Otherwise: print an error message, and if the layer doesn't exist     
 ;   make it with the default properties.                                  
 ; 
      (progn
           (prompt (strcat "** No data for layer " lanam
                           ": see Malaya.lsp **"))
           (if (null (tblsearch "layer" lanam))
               (command "layer" "make" lanam ""))))
 ; Ŀ
 ;   If the layer exists (and if we are here it should) make it current.   
 ; 
  (if (tblsearch "layer" lanam) (command "layer" "set" lanam ""))
 ; Ŀ
 ;   Reset everything.                                                     
 ; 
  (setvar "regenmode" regg)
  (setvar "expert" exp)
 (princ))
 ; Ŀ
 ;   Subroutine Malaya end.                                                
 ; 

 ; Ŀ
 ;   Subroutine Prlist: make the layer names and desired properties list.  
 ;   Takes no arguments, returns a list.                                   
 ;                                                                         
 ;   The sublist format is:  '(Layername colournumber linetype)            
 ;   Notes:  1. Colour names won't work, you must use numbers.             
 ;           2. For those like myself who aren't always awake, you can't   
 ;              set the colour of a layer to "Bylayer".                    
 ;           3. Less obviously, you can't use "Byblock" (0) either.        
 ;                                                                         
 ;   Malaya will use the first copy of a name which it finds, so if your   
 ;   changes don't seem to work look for a layer with the same name        
 ;   higher up in the file.                                                
 ;   The layer name must be in lower case if you intend to use fixla.      
 ;                                                                         
 ;   Colour number equivalents:                                            
 ;                                                                         
 ;      Red        1                                                       
 ;      Yellow     2                                                       
 ;      Green      3                                                       
 ;      Cyan       4           Actual colour rendition may vary.           
 ;      Blue       5           Please consult your retailer.               
 ;      Magenta    6                                                       
 ;      White      7                                                       
 ;      Dark gray  8                                                       
 ;                                                                         
 ; 
 (DEFUN PRLIST ()
  (list '("0"                   7    "continuous")
 ; Ŀ
 ;   Leading 4s - I'm sure this made sense to someone.                     
 ;   In some cases the suffix number is supposed to indicate the layer     
 ;   colour - notice entropy taking a hand here.                           
 ; 
        '("4-block-1"           2    "continuous")
        '("4-block-2"           1    "continuous")
        '("4-block-3"           3    "continuous")
        '("4-dim-1"             2    "continuous")
        '("4-equip-0"           7    "continuous")  
        '("4-equip-1"           2    "continuous")
        '("4-equip-2"           1    "continuous")
        '("4-gnd-3"             3    "c13")   
        '("4-gndsym-1"          2    "continuous")         
        '("4-hatch-0"           7    "continuous")
        '("4-hatch-4"           4    "continuous")
        '("4-line-1"            2    "continuous")
        '("4-line-3"            3    "ph1")  
        '("4-text-1"            2    "continuous")
        '("4-text-2"            1    "continuous")
        '("4-text-3"            3    "continuous")
        '("4-tray-2"            1    "continuous")
        '("4-wire-0"            7    "da4")
        '("4-wire-1"            2    "continuous")
        '("4-wire-3"            3    "continuous")
 ; Ŀ
 ;   Leading letters - obviously relics from an earlier time.              
 ; 
        '("a-revise"            11   "continuous")
        '("a-titleblock"        13   "continuous")
        '("c-building"          1    "continuous")
        '("t-topo"              3    "continuous")
 ; Ŀ
 ;   Leading Es with a dash - typically someone doing work for Encana.     
 ; 
        '("e-background"        5    "continuous")
        '("e-bom"               2    "continuous")
        '("e-busbar"            15   "continuous")   ; duplicate
;       '("e-busbar"            4    "continuous")   ; Rangeland
        '("e-cable"             3    "continuous")   ; Rangeland
        '("e-cableug"           15   "dashed")
        '("e-cable-ug"          3    "dashed")       ; Rangeland
        '("e-center"            3    "center")       ; duplicate
        '("e-dimension"         3    "continuous")
        '("e-eequip"            14   "continuous")   ; duplicate (hlf)
        '("e-equip"             2    "continuous")   ; gok - not Encana
        '("e-equipment"         2    "continuous")
        '("e-existing"          6    "continuous")
        '("e-field"             10   "dashed")
        '("e-frame"             13   "continuous")
        '("e-ground"            15   "center")       ; duplicate
        '("e-hidden"            6    "hidden")       ; duplicate
        '("e-instr"             10   "continuous")
        '("e-instrline"         6    "continuous")
        '("e-misc"              13   "continuous")   ; duplicate
        '("e-panel"             14   "continuous")
        '("e-shade"             1    "continuous")
        '("e-text1"             6    "continuous")   ; duplicate
        '("e-text2"             1    "continuous")   ; duplicate, was colour 3
        '("e-text3"             11   "continuous")   ; duplicate
        '("e-text4"             15   "continuous")
;       '("e-cable-tray"        165  "continuous")   ; Rangeland
;       '("e-center"            1    "continuous")   ; Rangeland
;       '("e-eequip"            13   "continuous")   ; Rangeland
;       '("e-ground"            4    "phantom")      ; Rangeland
;       '("e-hidden"            1    "hidden")       ; Rangeland
;       '("e-misc"              5    "continuous")   ; Rangeland
;       '("e-text1"             2    "continuous")   ; Rangeland
;       '("e-text2"             3    "continuous")   ; Rangeland
;       '("e-text3"             4    "continuous")   ; Rangeland
;       '("e-wire-field"        3    "dashed")       ; Rangeland
;       '("e-wire-panel"        3    "continuous")   ; Rangeland
 ; Ŀ
 ;   Leading Es with an underscore.                                        
 ; 
        '("e_busbar"            5    "continuous")
        '("e_cable"             4    "continuous")
        '("e_cable_control"     1    "continuous")
        '("e_cable_underground" 4    "hidden")
        '("e_cable_power"       4    "continuous")
        '("e_cln"               1    "center")
        '("e_communication"     2    "fenceline1")
        '("e_dim"               2    "continuous")
        '("e_areaclass"         3    "continuous")
        '("e_equipment"         3    "continuous")
        '("e_field_wiring"      122  "dashed")
        '("e_grounding"         4    "phantom")
        '("e_htc"               1    "continuous")
        '("e_heat_trace"        53   "continuous")
        '("e_key_boundary"      5    "border")
        '("e_mis"               7    "continuous")
        '("e_panel"             2    "continuous")
        '("e_panel_wiring"      42   "continuous")
        '("e_revision"          5    "continuous")
        '("e_symbols"           2    "continuous")
        '("e_txt"               2    "continuous")
 ; Ŀ
 ;   Some P&ID Layers.                                                     
 ; 
        '("flogen"              7    "continuous")
        '("flomain"             4    "continuous")
        '("flosec"              1    "continuous")
        '("floskidb"            2    "phantom")
 ; Ŀ
 ;   Layers named mostly after what they should contain.                   
 ; 
        '("areaclass"           3    "continuous")
        '("area"                6    "continuous")  ; pembina
        '("attrib"              7    "continuous")  ; pembina
        '("building"            1    "continuous")
        '("background"          8    "continuous") 
        '("base_equip"          253  "continuous")
        '("base_text"           41   "continuous")
        '("base_struct"         201  "continuous")
        '("base_bldg"           30   "continuous")
        '("berm"                6    "continuous")  ; pembina
        '("bldg"                1    "continuous")  ; pembina
        '("bom"                 2    "continuous") 
        '("border"              1    "continuous")
        '("border1"             4    "continuous")
        '("border2"             6    "continuous")
        '("building"            1    "continuous") 
        '("busbar"              4    "continuous") 
;       '("busbar"              5    "continuous")  ; Tridyne
        '("bus_bar"             4    "continuous")
        '("cable"               4    "continuous") 
        '("cable_control"       1    "continuous")  ; Tridyne
        '("cable_power"         4    "continuous")  ; Tridyne
        '("cable tray"          2    "continuous")  ; gok - Colleaux?
;       '("cable_tray"          2    "continuous")  ; Tridyne
        '("cable_tray"          5    "continuous")  ; pembina
        '("cable_tray_hatch"    4    "continuous")  ; Tridyne
        '("cable_underground"   4    "hidden")      ; Tridyne
;       '("cable_underground"   4    "dashed")      ; also Tridyne
        '("cableug"             4    "dashed")
        '("cent"                2    "center") 
;       '("center"              6    "center")      ; Rocket
        '("class"               2    "continuous")  ; pembina
        '("cloud"               3    "continuous")  ; gok - used for tb
        '("communication"       2    "continuous")  ; Tridyne
        '("concabu"             4    "dashed")
;       '("cond_cable"          7    "continuous")
        '("contin"              2    "continuous")
        '("contour"             7    "continuous")  ; pembina
        '("dash"                5    "dashed")      ; Exergy?
        '("defpoints"           8    "continuous") 
;       '("dimension"           3    "continuous")
        '("dimension"           1    "continuous")  ; Tridyne
        '("equip"               4    "continuous") 
        '("eequip"              3    "continuous") 
        '("el-sym"              1    "continuous")  ; gok
        '("elec_text"           7    "continuous")
        '("elec_equip"          4    "continuous")
        '("elec_cond"           1    "continuous")
        '("elec_instr"          201  "continuous")
        '("elec_exist"          201  "continuous")
;       '("equipment"           2    "continuous")
        '("equipment"           3    "continuous")  ; Tridyne
        '("existing"            6    "continuous") 
        '("fadeout"             8    "continuous")
        '("fence"               3    "continuous")  ; pembina
        '("field"               1    "dashed") 
;       '("field"               62   "continuous")  ; Tridyne
        '("field_wiring"        6    "dashed")      ; pembina
        '("frame"               1    "continuous") 
        '("future"              6    "phantom2") 
        '("gnd_text"            21   "continuous")
        '("gnd_equip"           90   "continuous")
        '("gnd_tray"            183  "center")
        '("gnd_wire"            92   "center")
;       '("grid"                2    "dot")         ; reality
        '("grid"                7    "dot")         ; pembina
        '("ground"              4    "phantom")
        '("grounding"           4    "phantom")     ; Tridyne
        '("ground_wire"         1    "center")
        '("hatch"               6    "continuous")
;       '("hatch"               7    "continuous")  ; Tridyne
        '("heat_trace"          1    "continuous")  ; Tridyne
;       '("heat_trace"          4    "phantom")     ; also Tridyne
        '("hidden"              6    "hidden")
        '("htrace"              5    "continuous")
        '("hvac"                1    "continuous")  ; pembina
        '("instr"               2    "continuous")
;       '("instr"               6    "continuous")  ; pembina
        '("instrline"           6    "continuous")
        '("instr_text"          140  "continuous")
        '("instr_equip"         6    "continuous")
        '("instr_dcond"         150  "continuous")
        '("instr_acond"         61   "continuous")
        '("instr_xcond"         191  "continuous")
        '("instr_exist"         233  "continuous")
        '("light_equip"         31   "continuous")
        '("light_cond"          112  "continuous")
        '("light_text"          221  "continuous")
        '("matchline"           4    "phantom")
        '("matchlines"          4    "phantom")     ; Tridyne
        '("medium"              2    "continuous")
        '("misc"                1    "continuous")  ; Tridyne
;       '("misc"                7    "continuous")
        '("mview"               7    "continuous")  ; pembina
        '("notes"               2    "continuous")
;       '("panel"               7    "continuous")
        '("panel"               1    "continuous")  ; Catch (Jeff W.)
;       '("panel"               2    "continuous")  ; Tridyne
        '("panel_wiring"        7    "continuous")  ; pembina
        '("pile"                6    "hidden")      ; pembina
        '("pneu_text"           40   "continuous")
        '("pneu_equip"          131  "continuous")
        '("pneu_flow"           3    "continuous")
        '("pneu_tube"           200  "continuous")
        '("revise"              4    "continuous")
        '("road"                2    "continuous")  ; pembina
        '("rotulo"              7    "continuous")
        '("shade"               255  "continuous")
        '("symbol"              2    "continuous")
        '("symbols"             2    "continuous")
        '("table"               2    "continuous")
;       '("text"                1    "continuous")
        '("text"                2    "continuous")  ; Tridyne
        '("text1"               2    "continuous")
        '("text_1"              2    "continuous")  ; gok
        '("text2"               2    "continuous")
        '("text3"               5    "continuous")
        '("text4"               4    "continuous")
        '("title"               2    "continuous")
        '("titleblock"          7    "continuous")
        '("title_text"          2    "continuous")
        '("topo"                3    "continuous")
        '("tray"                2    "continuous")
        '("trace"               4    "continuous")
        '("tray_text"           52   "continuous")
        '("tray_elec"           122  "continuous")
        '("tray_instr"          241  "continuous")
        '("ugnd_elec"           73   "hidden")
        '("ugnd_instr"          43   "hidden")
        '("ugnd_duct"           231  "continuous")
        '("ugnd_text"           254  "continuous")
        '("ugnd_struct"         181  "continuous")
        '("ugnd_equip"          132  "continuous")
        '("ugtext"              2    "continuous")  ; ??
        '("viewport"            8    "continuous")
        '("wire"                6    "continuous")  ; pembina
        '("xref"                253  "continuous")  ; pembina
 ; Ŀ
 ;   Enerplus stuff.  I hope they appreciate this.                         
 ; 
        '("construction"        4    "continuous")
        '("enequip"             3    "continuous")
        '("rev"                 2    "continuous")
        '("txt-01"              2    "continuous")
        '("txt-02"              3    "continuous")
        '("txt-03"              5    "continuous")
 ; Ŀ
 ;   Exergy stuff.                                                         
 ;   Also Pembina.                                                         
 ; 
        '("center"              1     "center")
        '("cond_cable"          6     "continuous")
        '("dash"                5     "dashed")
        '("field_wiring"        6     "dashed")
        '("ground_wire"         1     "center")     ; Exergy is 4
        '("hidden"              6     "hidden")
        '("phantom"             4     "phantom")
        '("panel_wiring"        7     "continuous") ; Exergy is 6
        '("ug_cond_cbl"         6     "hidden")
        '("vport"               1     "continuous")
 ; Ŀ
 ;   Agrium.                                                               
 ;   The duplicates aren't strictly electrical and are commented out.      
 ; 
        '("boundary"            4     "phantom")
        '("bus"                 1     "continuous")
        '("const"               8     "continuous")
        '("dim"                 7     "continuous")
;       '("equip"               3     "continuous")  ; duplicate
        '("feeder"              2     "continuous")
        '("greyscale"           8     "continuous")
;       '("hatch"               7     "continuous")  ; duplicate
;       '("instr"               4     "continuous")  ; duplicate
        '("linecenter"          4     "center")
        '("linedash"            4     "dashed")
        '("linehidden"          4     "hidden")
        '("linehvy"             2     "continuous")
        '("linelite"            4     "continuous")
        '("linemed"             3     "continuous")
        '("linephantom"         4     "phantom")
        '("linexhvy"            1     "continuous")
        '("mfl"                 1     "continuous")
        '("sfl"                 6     "continuous")
 ;      '("symbol"              4     "continuous")  ; duplicate
        '("ttlhvy"              2     "continuous")
        '("ttllite"             4     "continuous")
        '("ttlmed"              3     "continuous")
        '("txthvy"              5     "continuous")
        '("txtlite"             7     "continuous")
        '("txtmed"              6     "continuous")
        '("txtxhvy"             1     "continuous")
        '("vplayer"             7     "continuous")))
 ; Ŀ
 ;   Subroutine Prlist end.                                                
 ; 

 ; Ŀ
 ;   Subroutine Splat - divide a text string into a list of substrings.    
 ;   Arguments: Sepchr, the field separator character.                     
 ;              Linn, the text string.                                     
 ;   Returns a list of field values, removes leading and trailing spaces.  
 ;                                                                         
 ;   Completely rewritten 11.19.2000.                                      
 ; 
 (DEFUN SPLAT (sepchr linn / pos len name1 strlst)
  (while (/= (strlen linn) 0)
 ; Ŀ
 ;   Find the first separator character, save everything before it into    
 ;   the Name1 variable, remove it from the start of the string Linn.      
 ; 
         (setq pos 1)
         (setq len (strlen linn))
         (while (and (/= (substr linn pos 1) sepchr)
                     (>= len pos))
                (setq pos (1+ pos)))
         (setq name1 (substr linn 1 (1- pos)))
         (setq linn (substr linn (1+ pos)))
 ; Ŀ
 ;   Remove spaces from the front and back of Name1.                       
 ; 
         (while (and (> (strlen name1) 0)
                     (= (substr name1 (setq len (strlen name1))) " "))
                (setq name1 (substr name1 1 (1- len))))
         (while (and (> (strlen name1) 0)
                     (= (substr name1 1 1) " "))
                (setq name1 (substr name1 2)))
 ; Ŀ
 ;   Add Name1 to the substring list Strlst.                               
 ; 
         (setq strlst (append strlst (list name1))))
 ; Ŀ
 ;   If the string contained no separator characters then Strlst will be   
 ;   nil, so return a list containing the original string.                 
 ; 
  (if (null strlst) (setq strlst (list linn)))
 strlst)
 ; Ŀ
 ;   Splat end.                                                            
 ; 

 ; Ŀ
 ;   Xlay (utility): set up xref layers.                                   
 ; 
 (DEFUN C:XLAY (/ xlist clay rewind isxlay lanam clay)
 ; Ŀ
 ;   Make the xlayer data list.  It might be a good idea to update this    
 ;   before trying to use Xlay.  Also bear in mind that the successful     
 ;   use of xrefs is dependent on a degree of consistency in layering      
 ;   which is not inevitably available.                                    
 ; 
  (setq xlist '(("10000LB"      8   "Thaw")
                ("1500LB"       3   "Thaw")
                ("150LB"        1   "Thaw")
                ("2000LB"       1   "Thaw")
                ("2500LB"       8   "Thaw")
                ("3000LB"       5   "Thaw")
                ("300LB"        2   "Thaw")))
 ; Ŀ
 ;   Now step through the layer data table and fix each one if required.   
 ; 
  (setq rewind T)
  (while (setq clay (tblnext "layer" rewind))
         (setq rewind ())
         (setq lanam (cdr (assoc 2 clay)))
         (setq lanam (strcase lanam t))
         (setq isxlay (if (= 16 (logand 16 (cdr (assoc 70 clay)))) T ()))
         (setq namend (cadr (splat "|" lanam)))
 ; Ŀ
 ;   If there a data sublist in xlist for the layer in question, and it    
 ;   is an xref dependent layer:                                           
 ;   If it is to be thawed, set the colour, if it is to be frozen do so.   
 ; 
         (if (and isxlay (setq laydat (assoc namend xlist)))
             (if (= (nth 2 laydat) "Thaw")
                 (command "layer" "colour" (cadr laydat) lanam "")
                 (command "layer" "freeze" lanam ""))))
 (princ))
 ; Ŀ
 ;   Xlay end.                                                             
 ; 

 ; Ŀ
 ;   Malaya.                                                               
 ;   This is just a string input function, so that the program can be      
 ;   called from the command line or by another routine.                   
 ; 
 (DEFUN C:MALAYA (/ lanam)
  (setvar "cmdecho" 0)
 ; Ŀ
 ;   Ask for the name of the desired current layer.                        
 ;   This is designed to be used as part of a menu so there is no prompt.  
 ; 
  (setq lanam (getstring))
 ; Ŀ
 ;   Call Malaya to do the work.                                           
 ; 
  (malaya lanam)
 (princ))